
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::MkDir - directory action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::MkDir;

use strict;

use File::Path;
use File::stat;

use PSGConf::Action;

our @ISA = qw(PSGConf::Action);



###############################################################################
###  constructor
###############################################################################

my %defaults = (
	mode	=> 0755,
	uid	=> 0,
	gid	=> 0
);

sub new
{
	my ($class, %opts) = @_;

	map {
		$opts{$_} = $defaults{$_}
			if (!exists($opts{$_}));
	} keys %defaults;

	return PSGConf::Action::new($class, %opts);
}


###############################################################################
###  check method
###############################################################################

sub check
{
	my ($self, $psgconf) = @_;
	my ($retval);

	if (! -d $self->{name})
	{
		$self->{changed} = 1;
		return 1;
	}

	### check for ownership or permission differences
	$self->{stat} = stat($self->{name});
	if (!defined($self->{stat}))
	{
		warn "\n\t!!! can't stat '$self->{name}': $!\n";
		return -1;
	}

	if (($self->{stat}->mode & 07777) != $self->{mode}
	    || $self->{stat}->uid != $self->{uid}
	    || $self->{stat}->gid != $self->{gid})
	{
		$self->{changed} = 1;
		return 1;
	}

	### no changes needed
	return 0;
}


###############################################################################
###  diff method
###############################################################################

sub diff
{
	my ($self) = @_;

	print "###############################################################################\n";
	print "### DIFF FOR $self->{name}\n";
	print "###          (original directory does not exist)\n"
		if (! -d $self->{name});
	print "###############################################################################\n";
	if (! -d $self->{name})
	{
		print "+ MKDIR: $self->{name}\n";
	}
	elsif (defined($self->{stat}))
	{
		if (($self->{stat}->mode & 07777) != $self->{mode})
		{
			printf "+ MODE CHANGE: %04o -> %04o\n",
			       $self->{stat}->mode & 07777,
			       $self->{mode};
		}

		if ($self->{stat}->uid != $self->{uid})
		{
			print '+ UID CHANGE: ' . $self->{stat}->uid
			      . ' -> ' . $self->{uid} . "\n";
		}

		if ($self->{stat}->gid != $self->{gid})
		{
			print '+ GID CHANGE: ' . $self->{stat}->gid
			      . ' -> ' . $self->{gid} . "\n";
		}
	}

	print "\n";
}


###############################################################################
###  do() method
###############################################################################

sub do
{
	my ($self, $psgconf) = @_;

	if (! -d $self->{name})
	{
		if (-l $self->{name}
		    || -e $self->{name})
		{
			if (!$self->{backup})
			{
				if (!unlink($self->{name}))
				{
					warn "\t!!! unlink('"
						     . $self->{name}
						     . "'): $!\n";
					return -1;
				}
			}
			elsif (! $psgconf->save_file($self->{name}, $self->{backup}))
			{
				return -1;
			}
		}

		mkpath($self->{name});
	}

	if ((!exists($self->{stat})
	     || ($self->{stat}->mode & 07777) != $self->{mode})
	    && !chmod($self->{mode}, $self->{name}))
	{
		warn "\t!!! chmod(\"$self->{name}\"): $!\n";
		return -1;
	}

	if ((!exists($self->{stat})
	     || $self->{stat}->uid != $self->{uid}
	     || $self->{stat}->gid != $self->{gid})
	    && !chown($self->{uid}, $self->{gid}, $self->{name}))
	{
		warn "\t!!! chown(\"$self->{name}\"): $!\n";
		return -1;
	}

	return 1;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::MkDir - Directory creation action class for PSGConf

=head1 SYNOPSIS

  use PSGConf::Action::MkDir;

  $psgconf->register_actions(
		PSGConf::Action::MkDir->new(
			'name'		=> '/path/to/dir',
			'mode'		=> 01777,
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::MkDir> module provides a B<PSGConf> action class
for creating a directory.

The B<PSGConf::Action::MkDir> class is derived from the B<PSGConf::Action>
class, but it defines/overrides the following methods:

=over 4

=item check()

Checks to see whether the directory exists and whether its permissions
and ownership are correct.

=item diff()

Prints a message indicating whether the directory needs to be created and
whether its permissions or ownership need to be changed.

=item do()

Creates the directory, optionally saving pre-existing directory.  Also sets
permissions and ownership.

=back

In addition to the attributes supported by the B<PSGConf::Action>
class, the B<PSGConf::Action::MkDir> class supports the following
attributes:

=over 4

=item I<mode>

The octal mode of the directory.  Defaults to 0755.

=item I<uid>

The numeric user id of the directory.  Defaults to 0.

=item I<gid>

The numeric group id of the directory.  Defaults to 0.

=item I<backup>

A boolean flag to indicate whether a backup copy of the existing directory
will be saved.  If enabled, existing directories will be saved under their
original name but with the prefix ".<backupext>".  Default is no
backup.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action>

=cut

