
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Action::UntarFile - canned untar action type for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::UntarFile;

use strict;

use PSGConf::Action;
use PSGConf::Util;

our @ISA = qw(PSGConf::Action);

###############################################################################
###  check method
###############################################################################
sub check
{
  my ($self, $psgconf) = @_;

  $self->{changed} = 1;
  
  return 1;
}

###############################################################################
###  do method
###############################################################################
sub do
{
  my ($self, $psgconf) = @_;
  my ($cmdstr) = $psgconf->udb_tar_cmd;

  if ($self->{compressed})
  {
    $cmdstr .= 	" --use-compress-program " . $psgconf->udb_gzip_cmd;
  }

  $cmdstr .= " -C " . $self->{name} . " -xpf " . $self->{tarball};

  if (&PSGConf::Util::RunCommand($cmdstr, 1))
  {      
    warn "\n\t!!! can't untar user environment in " . $self->{name} . "\n";
    return 0;
  }

  unlink "$self->{tarball}" if ($self->{delete});
}

###############################################################################
###  diff method
###############################################################################
sub diff
{
  my ($self, $psgconf) = @_;

  print "###############################################################################\n";
  print "### DIFF FOR $self->{name}\n";
  print "###############################################################################\n";

  my ($cmdstr) = $psgconf->udb_tar_cmd;

  if ($self->{compressed})
  {
    $cmdstr .= 	" --use-compress-program " . $psgconf->udb_gzip_cmd;
  }

  $cmdstr .= " -C " . $self->{name} . " -xpf " . $self->{tarball};

  print "+ $cmdstr\n";
  print "+ unlink \"$self->{tarball}\"\n" if ($self->{delete});
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::UntarFile - file untar action class for PSGConf

=head1 SYNOPSIS

  use PSGConf::Action::UntarFile;

  $psgconf->register_actions(
		PSGConf::Action::UntarFile->new(
		      'name'		=> '/path/to/directory',
		      'tarball'       => '/path/to/tarball',
                      'compressed'    => [0,1] 1 if the tarball is compressed,
		      'delete'        => [0,1] 1 if the tarball should be
					       deleted after the operation
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::UntarFile> module provides a B<PSGConf> action class
for untarring a tape archive file

The B<PSGConf::Action::UntarFile> class is derived from the
B<PSGConf::Action> class, but it defines/overrides the
following methods:

=over 4

=item check()

  Verifies that the tarball exists and is a valid tape archive.

=item diff()
  Prints a diff message for the directory that is to be updated

=item do()
  Untars the file into the specified directory, then deletes the
  tar file if necessary.

=back
    
In addition to the attributes supported by the B<PSGConf::Action>
class, the B<PSGConf::Action::UntarFile> class supports the following
attributes:

=over 4

=item I<name>

The name of the directory where the tar file is to be extracted.

=item I<tarball>

The tar file to be extracted.

=item I<compressed>

1 if the tar file is compressed (gziped), 0 if it is uncompressed.

=item I<delete>

1 if the tar file is to be deleted after the extraction.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::File>

=cut

