
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Control - base class for psgconf control types
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control;

use strict;

use PSGConf::Action::MkDir;
use PSGConf::Action::Remove;
use PSGConf::Action::Symlink;
use PSGConf::Action::TouchFile;
use PSGConf::Data::Boolean;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;
use PSGConf::Data::ConfigError;

###############################################################################
###  constructor
###############################################################################

sub new
{
	my ($class, $psgconf, %opts) = @_;
	my ($self, $caller, $directive);

	$self = \%opts;

	bless($self, $class);

	if ((!exists($self->{name}) || $self->{name} eq '')
	    && ! $self->{quiet})
	{

		$caller = (caller(1))[3]
			if (!defined($caller));

		die "PSGConf::Control->new(): no name attribute from caller $caller\n";
	}

	# Register all our data objects
	$psgconf->register_data ( 
		$self->{name} . '_enable'	=> PSGConf::Data::Boolean->new(),
		$self->{name} . '_packages'	=> PSGConf::Data::List->new()
	);

	if (!exists($self->{diff_command})) {
		$psgconf->register_data ( 
			diff_command	=> PSGConf::Data::String->new(
							value => '/usr/bin/diff -c'
						),
			hostname		=> PSGConf::Data::String->new(
							value => 'unknown'
						),
			symlink		=> PSGConf::Data::Hash->new(
							key_abspath => 1
						),
			mkdir		=> PSGConf::Data::List->new(),
			unlink		=> PSGConf::Data::List->new(),
			touch		=> PSGConf::Data::List->new(),
			die			=> PSGConf::Data::ConfigError->new(
							fatal => 1
						),
			warn			=> PSGConf::Data::ConfigError->new()
		);
	}
	
	# Register all our policies
	$psgconf->register_policy ($self,
			$self->{name} . '_add_packages'	=> '_policy_add_packages'
	);
	
	return $self;
}


###############################################################################
###  policy method
###############################################################################

sub _policy_add_packages
{
	my ($self, $psgconf) = @_;
	my ($enable, $packages);

	$enable = $self->{name} . '_enable';
  
	return
		if ($psgconf->data_obj($enable)->equals('no'));

	$packages = '$psgconf->' . $self->{name} . '_packages';

	map {
		$psgconf->pkg_install_list->{$_} = undef
			if (!exists($psgconf->pkg_install_list->{$_}));
	} @{$packages};
}

###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;
	my ($file);

	### add symlink actions
	foreach $file (keys %{$psgconf->symlink})
	{
		$psgconf->register_actions(
			PSGConf::Action::Symlink->new(
				name		=> $file,
				link_to		=> $psgconf->symlink->{$file}
			)
		);
	}

	### add mkdir actions
	foreach $file (@{$psgconf->mkdir})
	{
		$psgconf->register_actions(
			PSGConf::Action::MkDir->new(
				name		=> $file
			)
		);
	}

	### add unlink actions
	foreach $file (@{$psgconf->unlink})
	{
		$psgconf->register_actions(
			PSGConf::Action::Remove->new(
				name		=> $file
			)
		);
	}

	### add touch actions
	foreach $file (@{$psgconf->touch})
	{
		$psgconf->register_actions(
			PSGConf::Action::TouchFile->new(
				name		=> $file
			)
		);
	}
}


1;

__END__

=head1 NAME

PSGConf::Control - base class for PSGConf control types

=head1 SYNOPSIS

  use PSGConf::Control;

  our @ISA = qw(PSGConf::Control);

  sub new {
    my ($class, $psgconf, %opts) = @_;
    $self = \%opts;
    bless($self, $class);

    ...

    $self = PSGConf::Control->new($psgconf, %$self);

    return $self;
  }

=head1 DESCRIPTION

The B<PSGConf::Control> module provides a class that represents an control
in B<PSGConf>.

The B<PSGConf::Control> class is not intended to be used to directly
instantiate control objects, but it does support the following methods
for use in subclasses:

=over 4

=item new()

The constructor.  It can be passed a hash to set the object's
attributes.  The object will be created as a reference to this hash.

The following attributes are supported:

=over 4

=item I<name>

A B<PSGConf::Data::String> object to store the name of the subclass.
It will be used in naming all further data objects.

=item I<name_enable>

A B<PSGConf::Data::Boolean> object to determine whether or not the
control module is to do any work.  In future revisions, this will
be defined to expicitly turn on or off the item it is controlling.
I<name> is to be replaced with the name that the subclass provides.

=item I<name_packages>

A B<PSGConf::Data::List> object of packages that may need to be installed
if this Control module is to be used.  I<name> is to be replaced with the 
name that the subclass provides.

=item I<diff_command>

A B<PSGConf::Data::String> object that contains the C<diff> command to
be used by the B<PSGConf::Action::File> object's diff() method.  The
default is C</usr/bin/diff -c>.

=item I<hostname>

A B<PSGConf::Data::String> object that contains the system's hostname.

=item I<mkdir>

A B<PSGConf::Data::List> object are a list of directories to be created 
on the system.

=item I<unlink>

A B<PSGConf::Data::List> object are a list of files to be deleted on 
the system.

=item I<touch>

A B<PSGConf::Data::List> object are a list of files to be touched 
on the system.

=item I<symlink>

A B<PSGConf::Data::Hash> object that contains a list of symlinks to be
created on the system.  The hash key is the absolute path of the
symlink, and the value is what the link should point to.

=item I<die>

A B<PSGConf::Data::ConfigError> object that, if set, will cause an
immediate configuration processing failure.

=item I<warn>

A B<PSGConf::Data::ConfigError> object that, if set, will cause a
warning message to be emitted.

=back

The constructor also registers the following policy method:

=over 4

=item I<name_add_package>

If I<name_enable> is defined, adds C<name_packages> to the 
I<pkg_install_list> object, which is provided by the 
B<PSGConf::Control::Packages> module.

=back

Subclasses should always define the following methods:

=over 4

=item decide()

Decide what actions need to be registered for this control object.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::File>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::Symlink>

L<PSGConf::Action::TouchFile>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::List>

L<PSGConf::Data::Hash>

L<PSGConf::Data::String>

L<PSGConf::Data::ConfigError>

=cut

