
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  AnonFTP.pm - anonymous FTP module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::AnonFTP;

use strict;

use File::stat;
use Unix::Mknod qw(major minor);

use PSGConf::Action::MkDir;
use PSGConf::Action::MkNod;
use PSGConf::Action::Symlink;
use PSGConf::Action::CopyFile;
use PSGConf::Action::CreateFile;
use PSGConf::Action::GenerateFile::Literal;
use PSGConf::Action::GenerateFile::etc_passwd;
use PSGConf::Action::GenerateFile::ftpaccess;
use PSGConf::Action::Remove;
use PSGConf::Action::svcs::setprop;
use PSGConf::Data::Boolean;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;

use PSGConf::Util;

###############################################################################
###  policy methods
###############################################################################

### ftp server name should default to hostname
sub _policy_default_servername
{
	my ($self, $psgconf) = @_;
	my ($tmp);

	return
		if (! $psgconf->anon_ftp_enable);

	$psgconf->data_obj('anon_ftp_server_name')->set($psgconf->hostname)
		if (!defined($psgconf->anon_ftp_server_name));

	$tmp = 'ftpadmin@' . $psgconf->anon_ftp_server_name;
	$psgconf->anon_ftp_options->{email} = $tmp
		if (!exists($psgconf->anon_ftp_options->{email}));
}


### uploads should be disabled by default
sub _policy_default_uploads
{
	my ($self, $psgconf) = @_;
	my ($tmp);

	return
		if (! $psgconf->anon_ftp_enable);

	### no uploading allowed by default
	$tmp = $psgconf->anon_ftp_dir . ' *';
	$psgconf->anon_ftp_upload->{$tmp} = 'no'
		if (!exists($psgconf->anon_ftp_upload->{$tmp}));
}


### create ftp user
sub _policy_add_user
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->anon_ftp_enable);

	$psgconf->group_info->{ftp} = {}
		if (!exists($psgconf->group_info->{ftp}));
	$psgconf->user_info->{ftp} = {
		group		=> 'ftp',
		passwd		=> $psgconf->passwd_token,
		home		=> $psgconf->anon_ftp_dir,
		gecos		=> 'Anonymous FTP',
		shell		=> '/bin/false'
	}
		if (!exists($psgconf->user_info->{ftp}));
}


### create ftp aliases
sub _policy_add_sendmail_aliases
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->anon_ftp_enable);

	$psgconf->sendmail_aliases->{ftp} = 'root'
		if (!exists($psgconf->sendmail_aliases->{ftp}));
	$psgconf->sendmail_aliases->{ftpadmin} = 'root'
		if (!exists($psgconf->sendmail_aliases->{ftpadmin}));
}


### add inetd entry
sub _policy_add_inetd_entry
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->anon_ftp_enable);

	$psgconf->inetd->{'ftp/tcp'} = {
		server		=> $psgconf->anon_ftp_ftpd_path,
		server_args	=> '-a -d -l -r ' . $psgconf->anon_ftp_dir
	}
		if (!exists($psgconf->inetd->{'ftp/tcp'}) &&
			$psgconf->platform !~ /solaris10/o );
}

sub _enable_rc_scripts
{
     my ($self, $psgconf) = @_;

     $psgconf->rc_scripts->{ftp}->{state} = 'enable'
          if ( $psgconf->anon_ftp_enable && 
		exists $psgconf->rc_scripts->{ftp} );
}



### add TCP wrappers entry
sub _policy_add_tcpwrapper_entry
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->anon_ftp_enable);

	$psgconf->data_obj('tcp_wrappers')->insert_row(
		{ 1 => 'all' },
		[ 'in.ftpd', 'all', 'allow' ]
	)
		if (! $psgconf->data_obj('tcp_wrappers')->find_row(
					{ 0 => qr/\bin.ftpd\b/ }));
}


### add syslog entry for "ftp" facility
### (Linux and *BSD only - other platforms don't have this facility)
sub _policy_add_syslog
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->anon_ftp_enable);

	return
		if ($psgconf->platform !~ m/linux|bsd/);

	$psgconf->syslog->{ftp} = 'ftp.info'
		if (!exists($psgconf->syslog->{ftp}));
}


### request the ftpd packages
sub _policy_add_packages
{
	my ($self, $psgconf) = @_;

	PSGConf::Util::_add_packages (
		$psgconf,
		$psgconf->anon_ftp_enable,
		@{$psgconf->anon_ftp_packages}
	);
}


###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;
	my ($file, $dest, $mode, $uid, $gid, $conversions, $userinfo);

	### if we're not enabled, remove ftpaccess and return
	if (!defined($psgconf->anon_ftp_enable))
	{
		$psgconf->register_actions(
			PSGConf::Action::Remove->new(
				name	=> $psgconf->anon_ftp_cfg_dir . '/ftpaccess'
			)
		);
		return;
	}

     ### add fake /etc/passwd file with just the anon ftp account.
	%{$userinfo->{ftp}} = %{$psgconf->user_info->{ftp}};
	$userinfo->{ftp}->{home} = '/';

	map {
		$conversions .= $_ . ":" . $psgconf->anon_ftp_conversions->{$_} . "\n"
	} keys %{$psgconf->anon_ftp_conversions};

	### instantiate ftpaccess file and ~ftp tree
	$psgconf->register_actions(
		PSGConf::Action::GenerateFile::ftpaccess->new(
			name		=> $psgconf->anon_ftp_cfg_dir . '/ftpaccess',
			description	=> 'FTP server configuration file',
			classes		=> $psgconf->anon_ftp_class,
			autogroups	=> $psgconf->anon_ftp_autogroup,
			limits		=> $psgconf->anon_ftp_limit,
			options		=> $psgconf->anon_ftp_options,
			messages	=> $psgconf->anon_ftp_message,
			readmes		=> $psgconf->anon_ftp_readme,
			uploads		=> $psgconf->anon_ftp_upload,
			literal_config	=> $psgconf->anon_ftp_literal
		),
		PSGConf::Action::MkDir->new(
			name		=> $psgconf->anon_ftp_dir,
			mode		=> 0555
		),
		(map {
			$file = $psgconf->anon_ftp_chroot_files->{$_};
			if ( exists $file->{location} ) {
				$dest = $psgconf->anon_ftp_dir . $file->{location};
			} else {
				$dest = $psgconf->anon_ftp_dir . $_;
			}
			$uid = ( exists $file->{owner} )? 
				$psgconf->user_info->{$file->{owner}}->{uid}:
				0;
			$gid = ( exists $file->{group} )?
				$psgconf->user_info->{$file->{group}}->{gid}:
				0;

			### If the source file does not exist, assume you
			### want to create a directory.
			if ( -d $_ || ! -e $_ ) {
				$mode = (exists $file->{mode})?  $file->{mode}: 0111;
				PSGConf::Action::MkDir->new(
					name		=> $dest,
					mode		=> $mode,
					uid		=> $uid,
					gid		=> $gid
				);
			} elsif ( -l $_ ) {
				PSGConf::Action::Symlink->new(
					name		=> $dest,
					link_to	=> $_
				);
			} elsif ( -b $_ || -c $_ ) {
				my ($st) = stat $_;
				$mode = (exists $file->{mode})?  $file->{mode}: 0644;
				PSGConf::Action::MkNod->new(
					name		=> $dest,
					mode		=> $mode,
					type		=> (-b $_)? 'b': 'c',
					major	=> major($st->rdev),
					minor	=> minor($st->rdev)
				);
			} elsif ( -f $_ ) {
				$mode = (exists $file->{mode})?  $file->{mode}: 0444;
				PSGConf::Action::CopyFile->new(
					name		=> $dest,
					copy_from	=> $_,
					mode		=> $mode,
					uid		=> $uid,
					gid		=> $gid
				);
			}
		} keys %{$psgconf->anon_ftp_chroot_files}),
		(map {
			PSGConf::Action::GenerateFile::Literal->new(
				name			=> $psgconf->anon_ftp_dir .
								$psgconf->anon_ftp_cfg_dir .
								'/ftp-banners/' . $_,
				comment_str	=> undef,
				content		=> $psgconf->anon_ftp_banners->{$_}
			)
		} sort keys %{$psgconf->anon_ftp_banners}),
		PSGConf::Action::GenerateFile::Literal->new(
			name		=> $psgconf->anon_ftp_dir .
						$psgconf->anon_ftp_cfg_dir .
						'/ftpconversions',
			content	=> $conversions
		),
		PSGConf::Action::GenerateFile::etc_passwd->new(
			'name'         => $psgconf->anon_ftp_dir . '/etc/passwd',
			'comment_str'  => undef,
			'mode'         => 0444,
			'sort_func'	=> sub { $a cmp $b },
			'user_info'    => $userinfo,
			'passwd_token' => $psgconf->passwd_token,
			'add_passwords'     => ($psgconf->platform =~ m/hpux/),
		),
		PSGConf::Action::CreateFile->new(
			'name'         => $psgconf->anon_ftp_dir . '/var/adm/wtmpx',
		)
	);


	if ( $psgconf->platform =~ /solaris10/o ) {
		$psgconf->register_actions(
			PSGConf::Action::svcs::setprop->new(
				name           => 'enable FTP startup options',
				FMRI           => 'svc:/network/ftp',
				property       => 'inetd_start/exec',
				value          => 
					'"' . $psgconf->anon_ftp_ftpd_path .
					' -a -d -l -r ' .  $psgconf->anon_ftp_dir . '"'
			)
		);
	}
}


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		anon_ftp_autogroup	=> PSGConf::Data::Hash->new(),
		anon_ftp_banners	=> PSGConf::Data::Hash->new(),
		anon_ftp_conversions	=> PSGConf::Data::Hash->new(),
		anon_ftp_class		=> PSGConf::Data::List->new(),
		anon_ftp_dir		=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/services/ftp'
					),
		anon_ftp_enable		=> PSGConf::Data::Boolean->new(),
		anon_ftp_packages		=> PSGConf::Data::List->new(),
		anon_ftp_limit		=> PSGConf::Data::List->new(),
		anon_ftp_literal	=> PSGConf::Data::String->new(),
		anon_ftp_cfg_dir	=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/etc'
					),
		anon_ftp_ftpd_path	=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/usr/sbin/in.ftpd'
					),
		anon_ftp_message	=> PSGConf::Data::Hash->new(),
		anon_ftp_options	=> PSGConf::Data::Hash->new(),
		anon_ftp_chroot_files	=> PSGConf::Data::Hash->new(
						value_type => 'HASH'
					),
		anon_ftp_readme		=> PSGConf::Data::Hash->new(),
		anon_ftp_server_name	=> PSGConf::Data::String->new(),
		anon_ftp_upload		=> PSGConf::Data::Hash->new(
						key_abspath => 1
					)
	);

	$psgconf->register_policy($self,
		anon_ftp_default_servername
				=> '_policy_default_servername',
		anon_ftp_default_uploads
				=> '_policy_default_uploads',
		anon_ftp_add_user
				=> '_policy_add_user',
		anon_ftp_add_sendmail_aliases
				=> '_policy_add_sendmail_aliases',
		anon_ftp_add_inetd_entry
				=> '_policy_add_inetd_entry',
		anon_ftp_add_tcpwrapper_entry
				=> '_policy_add_tcpwrapper_entry',
		anon_ftp_add_syslog
				=> '_policy_add_syslog',
		anon_ftp_add_packages
				=> '_policy_add_packages',
		anon_ftp_enable_rc_scripts
				=> '_enable_rc_scripts'
	);

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::AnonFTP - psgconf control class for anonymous FTP

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::AnonFTP

=head1 DESCRIPTION

The B<PSGConf::Control::AnonFTP> module provides a B<psgconf> control
object for configuring anonymous FTP using wu-ftpd.  It provides the
following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<anon_ftp_ftpd_path>

A B<PSGConf::Data::String> object containing the absolute path to the 
wu-ftpd binary.  The default is F</usr/local/sbin/in.ftpd>.

=item I<anon_ftp_autogroup>

A B<PSGConf::Data::Hash> object that specifies autogroup mappings.  The
hash key is the group name, and the value is a space-seperated list of
classes.

=item I<anon_ftp_banners>

A B<PSGConf::Data::Hash> object that specifies what FTP banner files
need to be created.  The hash key is the name of the file, and the value
is the file's contents.

=item I<anon_ftp_class>

A B<PSGConf::Data::List> object that contains a list of classes to
define in F<ftpaccess>.

=item I<anon_ftp_dir>

A B<PSGConf::Data::String> object that sets the path to the anonymous
FTP root.  It must be an absolute path.  If not set, it defaults to
F</services/ftp>.

=item I<anon_ftp_enable>

A B<PSGConf::Data::Boolean> object that enables anonymous FTP.  Default
is off.

=item I<anon_ftp_packages>

A B<PSGConf::Data::List> object lists what packages to install.

=item I<anon_ftp_limit>

A B<PSGConf::Data::List> object that contains a list of limits to define
in F<ftpaccess>.

=item I<anon_ftp_conversions>

A B<PSGConf::Data::Hash> object containing the contents of the
F<ftpconversions> file, where the key is the first 4 fields of
the ftpconversions file (the Strip prefix, Strip postfix, Addon
prefix, and Addon postfix) and the value is the rest of the line
in the file.

=item I<anon_ftp_cfg_dir>

A B<PSGConf::Data::String> object that contains the location of all the
ftpd configuration files.  Defaults to F</etc>.

=item I<anon_ftp_literal>

A B<PSGConf::Data::String> object that contains literal text to add to
F<ftpaccess>.  (This is a nasty hack, and should be avoided.)

=item I<anon_ftp_message>

A B<PSGConf::Data::Hash> object that specifies the C<message> directives
to add to F<ftpaccess>.  The hash key is the name of the message
file, and the value sets when the directive is triggered.

=item I<anon_ftp_options>

A B<PSGConf::Data::Hash> object that contains miscellaneous options for
F<ftpaccess>.  The hash key is the option name, and the value is
the option value.

=item I<anon_ftp_chroot_files>

A B<PSGConf::Data::Hash> object listing what files are needed to setup
the chroot'ed ftp environment.  The key is the file on the system,
and points to a hash with the following sub keys:

=over 4

=item I<location>

The location to put the file under I<anon_ftp_dir>.

=item I<mode>

The mode the file should have.

=back

=item I<anon_ftp_readme>

A B<PSGConf::Data::Hash> object that specifies the C<readme> directives
to add to F<ftpaccess>.  The hash key is the name of the F<README>
file, and the value sets when the directive is triggered.

=item I<anon_ftp_server_name>

A B<PSGConf::Data::String> object that contains the advertised server
name of the FTP server.  If not set, it defaults to the value of the
I<hostname> object, which is supplied by B<PSGConf::Control::Core>.

=item I<anon_ftp_upload>

A B<PSGConf::Data::Hash> object that specifies the C<upload> directives
to add to F<ftpaccess>.  The hash key is a string containing the
root directory, followed by a space, followed by the list of relative
directories (or a C<*>).  The value is C<no>, or C<yes> followed by
optional owner and permissions for the uploaded files.

=back

In addition, the constructor registers the following policy methods:

=over 4

=item I<anon_ftp_default_servername>

If the I<anon_ftp_server_name> object is unset, it is set to the value
of the I<hostname> object, which is provided by
B<PSGConf::Control::Core>.

If the I<anon_ftp_options> object does not contain a setting for the
C<email> option, it is set to C<ftpadmin@> followed by the value of the
I<anon_ftp_server_name> object.

=item I<anon_ftp_default_uploads>

If the I<anon_ftp_upload> object is unset, an entry is added for root
directory I<anon_ftp_dir>, subdirectory C<*>, value C<no>.

=item I<anon_ftp_add_user>

Adds entries to the I<user_info> and I<group_info> Data objects (supplied
by the B<PSGConf::Control::Users> module) for user and group C<ftp>.

=item I<anon_ftp_add_sendmail_aliases>

If mail aliases for C<ftp> and C<ftpadmin> do not exist, aliases are
created that point to C<root>.  This is done using the
I<sendmail_aliases> Data object, which is provided by the
B<PSGConf::Control::sendmail> module.

=item I<anon_ftp_enable_rc_scripts>

If there is an entry in the I<rc_scripts> hash for I<ftp>, then go ahead
and enable it.

=item I<anon_ftp_add_inetd_entry>

If there is no entry in the I<inetd> data object (provided by
B<PSGConf::Control::inetd>) for C<ftp/tcp>, create one that invokes the
binary specified by the I<anon_ftp_ftpd_path> data object with the
arguments C<-a -d -l -r anon_ftp_dir>.

=item I<anon_ftp_add_tcpwrapper_entry>

If the I<tcp_wrappers> object has no entry for C<in.ftpd>, add one with
the values C<in.ftpd>, C<all>, C<allow>.  This is done using
the I<tcp_wrappers> object, which is provided by
B<PSGConf::Control::TCPWrappers>.

=item I<anon_ftp_add_syslog>

Under Linux and *BSD, adds an F<ftp> entry to the I<syslog> data object
(supplied by B<PSGConf::Control::syslog>).  (On other platforms, there
is no C<ftp> syslog facility, so F<wu-ftpd> logs to the C<daemon>
facility.)

=item I<anon_ftp_add_packages>

Adds F<wu-ftpd> to the list of requested packages.  This
uses the I<pkg_install_list> config object supplied by
B<PSGConf::Control::Packages>.

=back

=item decide()

Instantiates and registers the following action objects:

=over 4

=item *

If I<anon_ftp_enable> is not set, it registers a
B<PSGConf::Action::Remove> object to remove F<ftpaccess> and
returns immediately.

=item *

Instantiates a B<PSGConf::Action::GenerateFile::ftpaccess> object to
create F<ftpaccess>.

=item *

Instantiates B<PSGConf::Action::GenerateFile::Literal> objects to create
each of the files listed in the I<anon_ftp_banners> object.  The files
are created in the F<anon_ftp_dir/ftp-banners> directory.

=item *

Instantiates B<PSGConf::Action::GenerateFile::Literal> objects to create
the ftpconversions file.

=item *

Instantiates B<PSGConf::Action::CreateFile> objects to create
the F<anon_ftp_dir/var/adm/wtmp> file.

=item *

Instantiates B<PSGConf::Action::GenerateFile::etc_passwd> objects to create
the I<anon_ftp_dir>/etc/passwd file.

=item *

If we are running on Solaris10, instantiates 
B<PSGConf::Action::svcs::setprop> objects to set the SMF properties for
ftpd to C<-a -d -l -r anon_ftp_dir>.

=item *

Instantiates B<PSGConf::Action::CopyFile> object for each file listed in
I<anon_ftp_chroot_files> object.  Instantiates B<PSGConf::Action::SymLink>
for each symlink, B<PSGConf::Action::MkNod> for each block or character
special file and B<PSGConf::Action::MkDir> for each directory or if the
file does not exist.

=back

=back

=head1 SEE ALSO

L<perl>

ftpaccess(5)

L<File::stat>

L<Unix::Mknod>

L<PSGConf>

L<PSGConf::Action::CopyFile>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::MkNod>

L<PSGConf::Action::Symlink>

L<PSGConf::Action::GenerateFile::ftpaccess>

L<PSGConf::Action::GenerateFile::etc_passwd>

L<PSGConf::Action::GenerateFile::Literal>

L<PSGConf::Action::Remove>

L<PSGConf::Action::CreateFile>

L<PSGConf::Control::inetd>

L<PSGConf::Control::Packages>

L<PSGConf::Control::sendmail>

L<PSGConf::Control::syslog>

L<PSGConf::Control::TCPWrappers>

L<PSGConf::Control::Users>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Hash>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<psgconf-intro>

=cut

