
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  Core.pm - core module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Control::Core;

use strict;

use PSGConf::Action::MkDir;
use PSGConf::Action::Remove;
use PSGConf::Action::Symlink;
use PSGConf::Action::CreateFile;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;
use PSGConf::Data::ConfigError;


###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;
	my ($file);

	### add symlink actions
	foreach $file (keys %{$psgconf->symlink})
	{
		$psgconf->register_actions(
			PSGConf::Action::Symlink->new(
				name		=> $file,
				link_to		=> $psgconf->symlink->{$file}
			)
		);
	}

	### add mkdir actions
	foreach $file (@{$psgconf->mkdir})
	{
		$psgconf->register_actions(
			PSGConf::Action::MkDir->new(
				name		=> $file
			)
		);
	}

	### add unlink actions
	foreach $file (@{$psgconf->unlink})
	{
		$psgconf->register_actions(
			PSGConf::Action::Remove->new(
				name		=> $file
			)
		);
	}

	### add create_file actions
	foreach $file (keys %{$psgconf->create_file})
	{
		$psgconf->register_actions(
			PSGConf::Action::CreateFile->new(
				name		=> $file
			)
		);
	}
}


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		diff_command	=> PSGConf::Data::String->new(
					value => '/usr/bin/diff -c'
				),
		etc_inet_dir	=> PSGConf::Data::String->new(
					value_abspath => 1,
					value => '/etc'
				),
		hostname	=> PSGConf::Data::String->new(),
		log_dir		=> PSGConf::Data::String->new(
					value_abspath => 1,
					value => '/var/log'
				),
		pidfile_dir	=> PSGConf::Data::String->new(
					value_abspath => 1,
					value => '/etc'
				),
		symlink		=> PSGConf::Data::Hash->new(
					key_abspath => 1
				),
		mkdir		=> PSGConf::Data::List->new(
					value_optional => 1
				),
		unlink		=> PSGConf::Data::List->new(),
		create_file	=> PSGConf::Data::Hash->new(
					value_optional => 1
				),
		die		=> PSGConf::Data::ConfigError->new(
					fatal => 1
				),
		warn		=> PSGConf::Data::ConfigError->new(),
	);

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::Core - psgconf control class for core system identity

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::Core

=head1 DESCRIPTION

The B<PSGConf::Control::Core> module provides a B<psgconf> control object
for configuring core system parameters, such as the system's hostname.
It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<diff_command>

A B<PSGConf::Data::String> object that contains the C<diff> command to
be used by the B<PSGConf::Action::File> object's diff() method.  The
default is C</usr/bin/diff -c>.

=item I<etc_inet_dir>

A B<PSGConf::Data::String> object that contains the absolute path to the
directory where network-related config files live.  The default is
F</etc>.

=item I<hostname>

A B<PSGConf::Data::String> object that contains the system's hostname.

=item I<log_dir>

A B<PSGConf::Data::String> object that contains the absolute path to the
system's log directory.  The default is F</var/log>.

=item I<pidfile_dir>

A B<PSGConf::Data::String> object that contains the absolute path to the
directory where pid files are stored on the system.  The default is
F</etc>.

=item I<mkdir>

A B<PSGConf::Data::List> object whose keys are a list of directories to be
created on the system.

=item I<unlink>

A B<PSGConf::Data::List> object whose keys are a list of files to be
deleted on the system.

=item I<create_file>

A B<PSGConf::Data::Hash> object whose keys are a list of files to be
created on the system.

=item I<symlink>

A B<PSGConf::Data::Hash> object that contains a list of symlinks to be
created on the system.  The hash key is the absolute path of the
symlink, and the value is what the link should point to.

=item I<die>

A B<PSGConf::Data::ConfigError> object that, if set, will cause an
immediate configuration processing failure.

=item I<warn>

A B<PSGConf::Data::ConfigError> object that, if set, will cause a
warning message to be emitted.

=back

=item decide()

For the paths specified by the I<symlink>, I<unlink>, I<create_file>, and
I<mkdir> data objects, instantiates the appropriate
B<PSGConf::Action::Symlink>, B<PSGConf::Action::Remove>,
B<PSGConf::Action::CreateFile>, and B<PSGConf::Action::MkDir> action
objects.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::File>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::Symlink>

L<PSGConf::Action::CreateFile>

L<PSGConf::Data::List>

L<PSGConf::Data::Hash>

L<PSGConf::Data::String>

L<PSGConf::Data::ConfigError>

L<psgconf-intro>

=cut

