
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  FreeBSD.pm - FreeBSD module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::FreeBSD;

use strict;

use PSGConf::Action::GenerateFile::EnvFile;
use PSGConf::Data::Hash;


###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;

	return
		if ($psgconf->platform !~ m/freebsd/);

	$psgconf->register_actions(
		PSGConf::Action::GenerateFile::EnvFile->new(
			'name'			=> '/etc/rc.conf',
			'description'		=> 'FreeBSD system config file',
			'quote_values'		=> 1,
			'requires_reboot'	=> 1,
			'vars'			=> $psgconf->rc_vars
		),
		PSGConf::Action::GenerateFile::EnvFile->new(
			'name'			=> '/etc/sysctl.conf',
			'description'		=> 'FreeBSD system control file',
			'requires_reboot'	=> 1,
			'vars'			=> $psgconf->sysctl_vars
		)
	);
}


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		'rc_vars'		=> PSGConf::Data::Hash->new(),
		'sysctl_vars'		=> PSGConf::Data::Hash->new(),
	);

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::FreeBSD - psgconf control class for FreeBSD-specific configuration

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::FreeBSD

=head1 DESCRIPTION

The B<PSGConf::Control::FreeBSD> module provides a B<psgconf> control object
for FreeBSD-specific configuration.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<rc_vars>

A B<PSGConf::Data::Hash> object that contains definitions for rc
variables to be added to F</etc/rc.conf>.

=item I<sysctl_vars>

A B<PSGConf::Data::Hash> object that contains definitions for system
control variables to be added to F</etc/sysctl.conf>.

=back

=item decide()

Under systems other than FreeBSD, returns without doing anything.
Otherwise, registers B<PSGConf::Action::GenerateFile::EnvFile> action
objects to create the F</etc/rc.conf> and F</etc/sysctl.conf> files.

=back

=head1 BUGS

The entries in I<rc_vars> that enable or disable RC scripts should be
integrated into B<PSGConf::Control::InitScripts> somehow.

=head1 SEE ALSO

L<perl>

rc.conf(5)

L<PSGConf>

L<PSGConf::Action::GenerateFile::EnvFile>

L<PSGConf::Data::Hash>

L<psgconf-intro>

=cut

