
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Control::Library::crle - class for configuring crle under Solaris
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::Library::crle;

use strict;
use PSGConf::Data::List;
use PSGConf::Data::Boolean;

###############################################################################
###  constructor
###############################################################################

sub _check_crle_cfg
{
	my ($self, $psgconf) = @_;
	my ($lib_list);
	local *CRLE;

	return 0 if ( ! $psgconf->crle_enable );

	if (! open(CRLE, $psgconf->crle_cmd . ' |'))
	{
		warn "\n\t!!! cannot run crle: $!\n";
		return -1;
	}

	grep ( /Default Library Path \(ELF\):	([^\s\t]*)/o && ( $lib_list = $1), <CRLE>);
	close CRLE;

	return 1 
		if ($lib_list ne join(':', @{$psgconf->crle_library_path_list}));

	return 0;
}

###############################################################################
###  constructor
###############################################################################

sub new
{
  my ($class, $psgconf) = @_;
  my ($self);
  
  $self = {};
  
  bless($self, $class);
  
  $self->{name} = 'crle';
  
  $psgconf->register_data(
	  'crle_library_path_list'	=> PSGConf::Data::List->new(),
	  'crle_cmd' => PSGConf::Data::String->new(
						value => '/usr/bin/crle'
					),
	  'crle_enable' => PSGConf::Data::Boolean->new()
  );

  return $self;
}

###############################################################################
###  decide
###############################################################################

sub decide
{
  my ($self, $psgconf) = @_;
  my ($list, $cmd);
      
  return
	if ( ! $psgconf->crle_enable );

  $list = $psgconf->crle_library_path_list;

  return if ( ! scalar @{$list} );

  $cmd = $psgconf->crle_cmd . ' -l ' . join(':', @{$psgconf->crle_library_path_list});
  
  $psgconf->register_actions(
	     PSGConf::Action::RunCommand->new(
			      name		=> $psgconf->crle_cmd . ' LD_LIBRARY_PATH modification',
			      command		=> $cmd,
			      check_func	=> \&_check_crle_cfg
	      )
  );
}

###############################################################################
###  documentation
###############################################################################

1;
__END__

=head1 NAME

PSGConf::Control::Library::crle - psgconf control class for controlling Solaris library locations

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::Library::crle

=head1 DESCRIPTION

The B<PSGConf::Control::Library::crle> module provides a B<psgconf> control object
for setting Solaris system-wide library load paths.
It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<crle_library_path_list>

A B<PSGConf::Data::List> object whose keys are the paths to directories to
be added to the library search path.

=item I<crle_enable>

A B<PSGConf::Data::Boolean> object to enable/disable this feature
    
=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::RunCommand>

L<PSGConf::Data::List>

L<psgconf-intro>

=cut
