
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Control::Library::ldconfig - class for configuring ldconfig on FreeBSD systems.
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::Library::ldconfig;

use strict;

use PSGConf::Action::GenerateFile::Literal;
use PSGConf::Action::MkDir;
use PSGConf::Action::RunCommand;
use PSGConf::Data::List;
use PSGConf::Data::String;
use PSGConf::Data::Boolean;

###############################################################################
###  constructor
###############################################################################

sub new
{
  my ($class, $psgconf) = @_;
  my ($self);
  
  $self = {};
  
  bless($self, $class);
  
  $self->{name} = 'ldconfig';
  
  $psgconf->register_data(
		  'ldconfig_cmd'	=> PSGConf::Data::String->new(
							value => '/sbin/ldconfig'
				),
		  'ldconfig_elf_path_list'	=> PSGConf::Data::List->new(),
		  'ldconfig_aout_path_list'	=> PSGConf::Data::List->new(),
		  'ldconfig_enable' => PSGConf::Data::Boolean->new(
							value => 'no'
				)
		  );
  
  return $self;
}

###############################################################################
###  constructor
###############################################################################

sub decide
{
  my ($self, $psgconf) = @_;
  my ($list, $dir);
      
  return
  	if ( ! $psgconf->ldconfig_enable );

  $list = $psgconf->ldconfig_elf_path_list;

  if ( scalar @{$list} ) {
	foreach $dir (@{$list}) {
  		$psgconf->register_actions(
			PSGConf::Action::MkDir->new(
					'name'		=> $dir
			)
		);
	}
  	$psgconf->register_actions(
			PSGConf::Action::GenerateFile::Literal->new(
				name	=> '/etc/ld-elf.so.conf',
				content  => join("\n", @{$list}) . "\n"
			),
			PSGConf::Action::RunCommand->new(
				name		=> 'ldconfig (elf)',
				command	=> $psgconf->ldconfig_cmd . ' -elf ' .
					join(' ', @{$list}),
				filename	=> '/etc/ld-elf.so.conf'
			)
  	);
  }

  if ($psgconf->platform =~ m/ix86/) {
  	$list = $psgconf->ldconfig_aout_path_list;

  	if ( scalar @{$list} ) {
		foreach $dir (@{$list}) {
  			$psgconf->register_actions(
				PSGConf::Action::MkDir->new(
						'name'		=> $dir
				)
			);
		}
  		$psgconf->register_actions(
				PSGConf::Action::GenerateFile::Literal->new(
					name	=> '/etc/ld.so.conf',
					content  => join("\n", @{$list}) . "\n"
				),
				PSGConf::Action::RunCommand->new(
					name		=> 'ldconfig (a.out)',
					command	=> $psgconf->ldconfig_cmd . ' -aout ' .
						join(' ', @{$list}),
					filename	=> '/etc/ld.so.conf'
				)
  		);
  	}
  }
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::Library::ldconfig - psgconf control class for controlling library locations using ldconfig command

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::Library::ldconfig

=head1 DESCRIPTION

The B<PSGConf::Control::Library::ldconfig> module provides a B<psgconf>
control object for setting library load paths.

It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<ldconfig_enable>

A B<PSGConf::Data::Boolean> object to decide whether or not to run
the C<ldconfig> command.  The default is no.

=item I<ldconfig_cmd>

A B<PSGConf::Data::String> object which defines the full path of the
F<ldconfig> command.  It defaults to F</sbin/ldconfig>.

=item I<ldconfig_elf_library_path>

A B<PSGConf::Data::List> object whose keys are the paths to directories to
be added to the library search path for elf based libraries.

=item I<ldconfig_aout_library_path>

A B<PSGConf::Data::List> object whose keys are the paths to directories to
be added to the library search path for a.out based libraries.

=back

=head1 SEE ALSO

L<perl>

L<ldconfig(8)>

L<PSGConf>

L<PSGConf::Action::GenerateFile::Literal>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::RunCommand>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<psgconf-intro>

=cut
