
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PAM.pm - PAM configuration module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::PAM;

use strict;

use PSGConf::Action::GenerateFile::pam_conf;
use PSGConf::Action::GenerateFile::pam_file;
use PSGConf::Data::Boolean;
use PSGConf::Data::List;
use PSGConf::Data::Hash;
use PSGConf::Util;


###############################################################################
###  policies
###############################################################################

sub _policy_add_packages
{
     my ($self, $psgconf) = @_;

     PSGConf::Util::_add_packages(
          $psgconf,
          $psgconf->pam_enable,
          @{$psgconf->pam_packages}
     );
}

###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->pam_enable);

	if ($psgconf->pam_directory)
	{
		map {
			$psgconf->register_actions(
				PSGConf::Action::GenerateFile::pam_file->new(
					'name'		=> "/etc/pam.d/$_",
					'description'	=> "PAM configuration for $_",
					pam_file	=> $psgconf->pam_conf->{$_}
				)
			);
		} sort keys %{$psgconf->pam_conf};
	}
	else
	{
		$psgconf->register_actions(
			PSGConf::Action::GenerateFile::pam_conf->new(
				'name'		=> '/etc/pam.conf',
				'description'	=> 'PAM configuration file',
				pam_conf	=> $psgconf->pam_conf
			)
		);
	}
}


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		'pam_conf'		=> PSGConf::Data::Hash->new(
						'value_type' => 'ARRAY'
					),
		'pam_enable'		=> PSGConf::Data::Boolean->new(),
		'pam_packages'		=> PSGConf::Data::List->new(),
		'pam_directory'		=> PSGConf::Data::Boolean->new()
	);

     $psgconf->register_policy($self,
          pam_add_packages   => '_policy_add_packages'
     );

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::PAM - psgconf control class for PAM configuration

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::PAM

=head1 DESCRIPTION

The B<PSGConf::Control::PAM> module provides a B<psgconf> control object
for configuring PAM.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<pam_conf>

A B<PSGConf::Data::Hash> object containing the PAM configuration for
each service.  The key is the name of the PAM service, and the value is
a reference to a list of entries for that service.

=item I<pam_enable>

A B<PSGConf::Data::Boolean> object that determines whether PAM should be
configured.  The default is no.

=item I<pam_packages>

A B<PSGConf::Data::List> object that lists all packages to install.

=item I<pam_directory>

A B<PSGConf::Data::Boolean> object that determines whether PAM is
configured by generating individual files in the F</etc/pam.d> directory.
The default is to configure PAM by generating F</etc/pam.conf>.

=back

=item decide()

If I<pam_enable> is set, registers the appropriate
B<PSGConf::Action::GenerateFile> objects based on the value of
I<pam_directory>.

=back

=head1 SEE ALSO

L<perl>

pam.conf(4)

L<PSGConf>

L<PSGConf::Action::GenerateFile>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::List>

L<PSGConf::Data::Hash>

L<PSGConf::Util>

L<psgconf-intro>

=cut

