
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PostgreSQL.pm - PostgreSQL module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::PostgreSQL;

use strict;

use PSGConf::Data::Boolean;
use PSGConf::Data::List;

use PSGConf::Util;


###############################################################################
###  policy methods
###############################################################################

sub _policy_add_packages
{
	my ($self, $psgconf) = @_;
          
	PSGConf::Util::_add_packages(
		$psgconf,
		$psgconf->postgresql_enable,
		@{$psgconf->postgresql_packages}
	);
}


sub _policy_add_kernel_params
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->postgresql_enable);

	if ($psgconf->platform =~ m/solaris/)
	{
		$psgconf->system_vars->{'shmsys:shminfo_shmmax'} = 2097152
			if (!exists($psgconf->system_vars->{'shmsys:shminfo_shmmax'}));
		$psgconf->system_vars->{'shmsys:shminfo_shmmin'} = 1
			if (!exists($psgconf->system_vars->{'shmsys:shminfo_shmmin'}));
	}
}


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		postgresql_enable	=> PSGConf::Data::Boolean->new(
								value => 0
							),
		postgresql_packages	=> PSGConf::Data::List->new()
	);

	$psgconf->register_policy($self,
		pgsql_add_kernel_params	=> '_policy_add_kernel_params',
		pgsql_add_packages	=> '_policy_add_packages'
	);

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::PostgreSQL - psgconf control class for PostgreSQL

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::PostgreSQL

=head1 DESCRIPTION

The B<PSGConf::Control::PostgreSQL> module provides a B<psgconf> control object
for configuring PostgreSQL.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers a B<PSGConf::Data::Boolean> data object called
I<postgresql_enable> that determines whether PostgreSQL will be
installed and configured.

The constructor also registers the following policy methods:

=over 4

=item I<pgsql_add_packages>

Adds C<postgresql_packages> to the I<pkg_install_list> data object (which is
supplied by the B<PSGConf::Control::Packages> module).

=item I<pgsql_add_kernel_params>

Under Solaris, adds entries to the I<system_vars> object (provided by
B<PSGConf::Control::Solaris>) to set C<shmsys:shminfo_shmmax> to
C<2097152> and C<shmsys:shminfo_shmmin> to C<1>.

=back

=back

=head1 SEE ALSO

L<perl>

postmaster(1)

L<PSGConf>

L<PSGConf::Control::Packages>

L<PSGConf::Control::Solaris>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::List>

L<PSGConf::Util>

L<psgconf-intro>

=cut

