
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  RHN.pm - Red Hat Network module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::RHN;

use strict;

use PSGConf::Action::GenerateFile::EnvFile;
use PSGConf::Action::RPMImportGPGKey;
use PSGConf::Data::Boolean;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;

use PSGConf::Util;


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		rhn_enable		=> PSGConf::Data::Boolean->new(
							value => 0
						),
		rhn_config		=> PSGConf::Data::Hash->new(),
		pkg_up2date_cmd	=> PSGConf::Data::String->new(
							value => "/usr/sbin/up2date"
						),
		gpg_cmd			=> PSGConf::Data::String->new(
							value => "/usr/bin/gpg"
						),
		rhn_gpg_keys		=> PSGConf::Data::List->new(),
		gpg_keyserver		=> PSGConf::Data::String->new(
							value => "hkp://subkeys.pgp.net"
						),
		rhn_packages		=> PSGConf::Data::List->new()
	);

	$psgconf->register_policy($self,
		rhn_add_packages	=> '_policy_add_packages',
		rhn_add_rc_scripts	=> '_policy_add_rc_scripts'
	);


	return $self;
}


###############################################################################
###  policy methods
###############################################################################

sub _policy_add_packages
{
     my ($self, $psgconf) = @_;

	PSGConf::Util::_add_packages(
		$psgconf,
          $psgconf->rhn_enable,
     	@{$psgconf->rhn_packages}
	);
}

### create RC script
sub _policy_add_rc_scripts
{                          
     my ($self, $psgconf) = @_;
          
     $psgconf->rc_scripts->{'rhnsd'}->{state} = 'enable'
          if ( $psgconf->rhn_enable );
}


###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;
	my ($key);

	return
		if (! $psgconf->rhn_enable);

	$psgconf->register_actions(
		PSGConf::Action::GenerateFile::EnvFile->new(
			name			=> '/etc/sysconfig/rhn/up2date',
			description	=> 'RHN configuration file',
			mode			=> 0600,
			vars			=> $psgconf->rhn_config
		)
	);

	foreach $key ( @{$psgconf->rhn_gpg_keys} ) {
		$psgconf->register_actions(
			PSGConf::Action::RPMImportGPGKey->new(
				name		=> "GPG KeyID $key",
				key		=> $key,
				rpm_cmd	=> $psgconf->pkg_rpm_cmd,
				gpg_cmd	=> $psgconf->gpg_cmd,
				keyserver	=> $psgconf->gpg_keyserver
			)
		);
	}
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::RHN - psgconf control class for RedHat Network.

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::RHN

=head1 DESCRIPTION

The B<PSGConf::Control::RHN> module provides a B<psgconf> control object
for configuring the RedHat Network and up2date client.  It supports the
following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<rhn_enable>

A B<PSGConf::Data::Boolean> object indicating whether RHN should be
configured.  Defaults to no.

=item I<rhn_config>

A B<PSGConf::Data::Hash> object containing the contents of the 
F</etc/sysconfig/rhn/up2date> file.

=item I<gpg_cmd>

A B<PSGConf::Data::String> object that determines the location of the
I<GnuPG> command.  Defaults to F</usr/bin/gpg>.

=item I<gpg_keyserver>

A B<PSGConf::Data::String> object that determines what GPG Key server
to fetch the keys from.  Defaults to F<hkp://subkeys.pgp.net>.

=item I<rhn_gpg_keys>

A B<PSGConf::Data::List> object containing the GPG keys to trust when
installing packages.

=item I<rhn_packages>

A B<PSGConf::Data::List> object containing the packages to install.

=item I<pkg_up2date_cmd>

A B<PSGConf::Data::String> object that determines the location of the
I<up2date> command.  The default is F</usr/sbin/up2date>.

=back

The constructor also registers the following policy methods:

=over 4

=item I<rhn_add_packages>

If I<rhn_enable> is set, adds the I<rhn_packages> to the 
I<pkg_install_list>.

=back

=item decide()

If I<rhn_enable> is set, registers the following action objects:

=over 4

=item *

Registers a B<PSGConf::Action::GenerateFile::EnvFile> object to create
F</etc/sysconfig/rhn>.

=item *

Registers a B<PSGConf::Action::RPMImportGPGKey> object to download and
install all the keys listsed in F<rhn_gpg_keys>.

=back

=back

=head1 SEE ALSO

L<up2date(8)>

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::EnvFile>

L<PSGConf::Action::RPMImportGPGKey>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Hash>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<psgconf-intro>

=cut

