
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Control::logadm - psgconf control class for controlling log rotation/pruning for Solaris
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::logadm;

use PSGConf::Action::GenerateFile::Literal;

use PSGConf::Data::Boolean;
use PSGConf::Data::Integer;
use PSGConf::Data::List;

###############################################################################
### policies
###############################################################################

sub _add_files
{
	my ($psgconf, $cmd, @files) = @_;
	my ($file, $fullcmd);

	# Setup the common arguments first.
	$fullcmd = " -z 0 -C " . $psgconf->logadm_file_cnt;
	$fullcmd .= " -a '" . $cmd . "'"
		if ( defined $cmd );

	foreach $file ( @files ) {

		# FIXME: Need to check to make sure the file is not already
		# in the list.
		if ( ! -f $file ) {
			$psgconf->data_obj('logadm_files')->add ( $file . ' -N' . $fullcmd );
		} else {
			$psgconf->data_obj('logadm_files')->add ( $file . $fullcmd );
		}
	}
}

sub _policy_add_syslog_files 
{
	my ($self, $psgconf) = @_;
	my (@logs);

	return
		if (!($psgconf->syslog_enable && $psgconf->logadm_enable));

	map {
		push @logs, $_ if ( -f $_ );
	} keys %{$psgconf->syslog};

	&_add_files ($psgconf, 
		"kill -HUP `cat " . $psgconf->pidfile_dir . "/syslog.pid`",
		@logs );
}

sub _policy_add_ftp_files 
{
     my ($self, $psgconf) = @_;

	return
		if (!($psgconf->anon_ftp_enable && $psgconf->logadm_enable));

	&_add_files ($psgconf, undef, ( $psgconf->log_dir . '/xferlog' ));
}

sub _policy_add_www_files 
{
     my ($self, $psgconf) = @_;
	my (@logs);

	return
		if (!($psgconf->www_enable && $psgconf->logadm_enable));

	@logs = (
		$psgconf->www_log_dir . '/suexec',
		$psgconf->www_log_dir . '/error_log',
		$psgconf->www_log_dir . '/access_log'
	);

	map { push @logs, $_; } values %{$psgconf->www_vh_access_log};

	push @logs, $psgconf->www_log_dir . '/ssl_engine_log'
		if ( %{$psgconf->www_vh_ssl} );

	push @logs, $psgconf->www_jk_logfile
		if ( $psgconf->www_jk_enable );

	###
	### FIXME: Need to add the bluestem and anyother logs not in
	### the stock psgconf modules.  This really screams to be put
	### into a method that other Control modules can call from w/i
	### one of their policies.
	###

	&_add_files ( $psgconf, 
		$psgconf->www_apachectl_path . " graceful > /dev/null 2>&1", 
		@logs);
}

###############################################################################
###
###  new() constructor
###
###############################################################################

sub new {
	my ($class, $psgconf) = @_;
	my ($self);
  
	$self = {};
  
	bless($self, $class);
  
	$self->{name} = 'logadm';

	$psgconf->register_data(
		'logadm_enable'	=> PSGConf::Data::Boolean->new(),
		'logadm_file_cnt'	=> PSGConf::Data::Integer->new(
							value => 14
						),
		'logadm_files'		=> PSGConf::Data::List->new()
	);

	### 
	### Rotate the syslog/ftp/apache logs.  Do not need to do the
	### TSM logs as dsmc does it for us (by setting the retention
	### in the config files).
	### 
	$psgconf->register_policy($self,
		logadm_add_syslog_files		=> '_policy_add_syslog_files',
		logadm_add_ftp_files		=> '_policy_add_ftp_files',
		logadm_add_www_files		=> '_policy_add_www_files'
	);

	return $self;
}

###############################################################################
###
###  decide() method
###
###  generate /etc/logadm.conf based on files specified in psg.conf
###
###############################################################################

sub decide {
	my ($self, $psgconf) = @_;
	my ($conffile);
	
	return
		if (! $psgconf->logadm_enable);

	### create /etc/logadm.conf
	$psgconf->register_actions(
		PSGConf::Action::GenerateFile::Literal->new(
			name		=> '/etc/logadm.conf',
			content	=> join("\n", @{$psgconf->logadm_files}) . "\n",
		)
	)
}

1;

__END__

=head1 NAME

PSGConf::Control::logadm - psgconf control class for controlling log rotation/pruning for Solaris

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::logadm

=head1 DESCRIPTION

The B<PSGConf::Control::logadm> module provides a B<psgconf> control object
for rotating system logs using the logadm facility.
It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<logadm_enable>

A B<PSGConf::Data::Boolean> object to enable/disable this feature

=item I<logadm_file_cnt>

A B<PSGConf::Data::Integer> object for the number of files we need to
keep.  The default is 14.

=item I<logadm_files>

A B<PSGConf::Data::List> object of the commands to use to configure
C<logadm>.

=back

The constructor also registers the following policy methods:

=over 4        

=item I<logadm_add_syslog_files>

Requests that the C<syslog> files be rotated.

=item I<logadm_add_ftp_files>

Requests that the C<anon_ftp> log files be rotated.

=item I<logadm_add_www_files>

Requests that the C<apache> log files be rotated.

=back

=item decide()

Registers a B<PSGConf::Action::GenerateFile::Literal> Action object to
generate the F</etc/logadm.conf> file.

=back

=head1 SEE ALSO

L<logadm(1m)>

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::Literal>

L<PSGConf::Data::List>

L<PSGConf::Data::Integer>

L<PSGConf::Data::Boolean>
    
L<psgconf-intro>

=cut
