
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  sendmail.pm - sendmail module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::sendmail;

use strict;

use File::Basename;

use PSGConf::Action::GenerateFile::sendmail_aliases;
use PSGConf::Action::GenerateFile::sendmail_mc;
use PSGConf::Action::GenerateFile::sendmail_cf;
use PSGConf::Action::GenerateFile::sendmail_trusted_users;
use PSGConf::Action::GenerateFile::sendmail_map;
use PSGConf::Action::MkDir;
use PSGConf::Action::RunCommand;
use PSGConf::Action::RestartDaemon;
use PSGConf::Data::Boolean;
use PSGConf::Data::List;
use PSGConf::Data::Hash;
use PSGConf::Data::Integer;
use PSGConf::Data::String;
use PSGConf::Util;


###############################################################################
###  policy methods
###############################################################################

### set up rc scripts
sub _policy_add_rc_scripts
{
	my ($self, $psgconf) = @_;
	my ($start_cmd);

	$psgconf->rc_scripts->{'sendmail'}->{state} = 'enable'
		if ( $psgconf->sendmail_enable );

	### set start command
	$start_cmd = $psgconf->sendmail_path . ' -bd -q20m \
			&& ' . $psgconf->rc_echo . ' " (MTA)\c" 
		if [ -s /etc/mail/submit.cf ]; then
			' . $psgconf->sendmail_path . ' -L sm-msp -Ac -q20m \
				&& ' . $psgconf->rc_echo . ' ", (MSP)\c" 
		fi';

	$start_cmd =~ s/-q20m/-qp/
		if ($psgconf->sendmail_use_persistent_queue_runners);

	$psgconf->rc_scripts->{'sendmail'}->{start_cmd} = $start_cmd;
}


### add smmsp user and group
sub _policy_add_user
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->sendmail_enable);

	$psgconf->group_info->{$psgconf->sendmail_msa_group} = {
		recommended_gid	=> $psgconf->sendmail_msa_gid
	}
		if (!exists($psgconf->group_info->{$psgconf->sendmail_msa_group}));
	$psgconf->user_info->{$psgconf->sendmail_msa_user} = {
		recommended_uid	=> $psgconf->sendmail_msa_uid,
		passwd		=> $psgconf->passwd_token,
		group		=> $psgconf->sendmail_msa_group,
		gecos		=> 'sendmail message submission program',
		home		=> '/var/spool/clientmqueue'
	}
		if (!exists($psgconf->user_info->{$psgconf->sendmail_msa_user}));
}


### install sendmail package
sub _policy_add_packages
{
	my ($self, $psgconf) = @_;

	PSGConf::Util::_add_packages(
		$psgconf,
		$psgconf->sendmail_enable,
		@{$psgconf->sendmail_packages}
	);
}


### set confPID_FILE option
sub _policy_default_pidfile
{
	my ($self, $psgconf) = @_;
	my ($smopts);

	return
		if (! $psgconf->sendmail_enable);

	$smopts = $psgconf->sendmail_options;

	$smopts->{'confPID_FILE'} = $psgconf->pidfile_dir . '/sendmail.pid'
		if (!exists($smopts->{'confPID_FILE'}));
}


### set default trusted-users file
sub _policy_default_ct_file
{
	my ($self, $psgconf) = @_;
	my ($smopts);

	return
		if (! $psgconf->sendmail_enable);

	$smopts = $psgconf->sendmail_options;

	if (%{$psgconf->sendmail_trusted_users}
	    && !exists($smopts->{'confCT_FILE'}))
	{
		$smopts->{'confCT_FILE'} = '-o /etc/mail/trusted-users';
		$psgconf->sendmail_features->{'use_ct_file'} = undef;
	}
}


my %_default_aliases = (
	postmaster	=> 'root',
	'MAILER-DAEMON'	=> 'postmaster',
	nobody		=> '/dev/null'
);

### set default aliases
sub _policy_default_aliases
{
	my ($self, $psgconf) = @_;
	my ($aliases);

	$aliases = $psgconf->sendmail_aliases;

	map {
		$aliases->{$_} = $_default_aliases{$_}
			if (!exists($aliases->{$_}));
	} keys %_default_aliases;
}


###############################################################################
###  decide() method
###############################################################################

sub _add_queue_dirs
{
	my ($self, $psgconf, $qdir) = @_;

	$psgconf->register_actions(
		PSGConf::Action::MkDir->new(
			name	=> $qdir,
			mode	=> $psgconf->sendmail_queue_mode,
			'uid' => $psgconf->sendmail_mta_user,
			'gid' => $psgconf->sendmail_mta_group
		),
		($psgconf->sendmail_queue_qf_subdirs
		 ? (map {
			PSGConf::Action::MkDir->new(
				name	=> "$qdir/$_",
				mode	=> $psgconf->sendmail_queue_mode,
				'uid' => $psgconf->sendmail_mta_user,
				'gid' => $psgconf->sendmail_mta_group
			)
		    } qw(qf df xf))
		 : ()
		),
	);
}

sub decide
{
	my ($self, $psgconf) = @_;
	my ($smopts, $smfeatures, $qgrps);
	my ($alias_file, $smmsp_gid, $smmsp_uid);
	my ($platform, $access_map, $mailertable, $map_type, $map_path);
	my ($num_queues, $qdir);

	return
		if (! $psgconf->sendmail_enable);

	$smopts = $psgconf->sendmail_options;
	$smfeatures = $psgconf->sendmail_features;
	$qgrps = $psgconf->sendmail_queue_groups;
	$num_queues = $psgconf->sendmail_num_queues_per_group;

	$alias_file = $smopts->{'ALIAS_FILE'};
	$alias_file =~ s|,.*||;

	$smmsp_uid = $psgconf->user_info->{'smmsp'}->{'uid'};
	$smmsp_gid = $psgconf->group_info->{'smmsp'}->{'gid'};

	foreach $qdir ($smopts->{'QUEUE_DIR'},
		       map { $_->{Path} }
		           grep { exists($_->{Path}) }
			       (values %$qgrps))
	{
		if (substr($qdir, -1, 1) ne '*')
		{
			$self->_add_queue_dirs($psgconf, $qdir);
			next;
		}

		map {
			$self->_add_queue_dirs(
				$psgconf,
				sprintf('%s%0*d',
					substr($qdir, 0, length($qdir) - 1),
					length("$num_queues"),
					$_)
			);
		} (1 .. $num_queues);
	}

	$psgconf->register_actions(
		PSGConf::Action::MkDir->new(
			'name'	=> $smopts->{'confHOST_STATUS_DIRECTORY'}
		)
	)
		if (exists($smopts->{'confHOST_STATUS_DIRECTORY'}));

	$psgconf->register_actions(
		PSGConf::Action::MkDir->new(
			'name'		=> '/var/spool/clientmqueue',
			'uid'		=> $smmsp_uid,
			'gid'		=> $smmsp_gid,
			'mode'		=> 0770
		),
		PSGConf::Action::GenerateFile::sendmail_mc->new(
			'name'		=> '/etc/mail/sendmail.mc',
			'comment_str'	=> 'dnl',
			'description'	=> 'sendmail configuration file',
			'ostype'		=> $psgconf->sendmail_ostype,
			'versionid'	=> 'sendmail.mc - generated by psgconf',
			'defines'		=> $smopts,
			'features'	=> $smfeatures,
			'masquerade_as'	=> $psgconf->sendmail_masquerade_as,
			'exposed_users'	=> $psgconf->sendmail_exposed_users,
			'mailers'		=> [ qw(local smtp) ],
			'queue_groups'	=> $psgconf->sendmail_queue_groups,
			'literal'		=> $psgconf->sendmail_literal
		),
		PSGConf::Action::GenerateFile::sendmail_cf->new(
			'name'		=> '/etc/mail/sendmail.cf',
			'comment_str'	=> undef,
			'user'		=> $psgconf->sendmail_mta_user,
			'mc_file'	=> '/etc/mail/sendmail.mc',
			'm4_command'	=> $psgconf->sendmail_m4_command,
			'sendmail_m4_dir' => $psgconf->sendmail_m4_dir
		),
		PSGConf::Action::GenerateFile::sendmail_mc->new(
			'name'		=> '/etc/mail/submit.mc',
			'comment_str'	=> 'dnl',
			'description'	=> 'MSP configuration file',
			'versionid'	=> 'submit.mc - generated by psgconf',
			'defines'	=> {
				confCF_VERSION		=> 'Submit',
				__OSTYPE__		=> '',
				_USE_DECNET_SYNTAX_	=> 1,
				confTIME_ZONE		=> 'USE_TZ',
				confDONT_INIT_GROUPS	=> 'true',
				confCT_FILE	=> '-o /etc/mail/trusted-users',
				confPID_FILE	=>  $psgconf->pidfile_dir . '/sm-client.pid'
			},
			'features'	=> {
				use_ct_file	=> undef
			},
			### the msp feature must come at the end,
			### regardless of sort order
			'literal'	=> 'FEATURE(`msp\')dnl'
		),
		PSGConf::Action::GenerateFile::sendmail_cf->new(
			'name'		=> '/etc/mail/submit.cf',
			'comment_str'	=> undef,
			'mc_file'	=> '/etc/mail/submit.mc',
			'user'		=> $psgconf->sendmail_mta_user,
			'm4_command'	=> $psgconf->sendmail_m4_command,
			'sendmail_m4_dir' => $psgconf->sendmail_m4_dir
		),
		PSGConf::Action::GenerateFile::sendmail_trusted_users->new(
			'name'		=> '/etc/mail/trusted-users',
			'description'	=> 'trusted users file',
			'trusted_users'	=> [ keys %{$psgconf->sendmail_trusted_users} ]
		),
		PSGConf::Action::GenerateFile::sendmail_aliases->new(
			'name'		=> $alias_file,
			'description'	=> 'Local mail aliases',
			'aliases'	=> $psgconf->sendmail_aliases
		),
		PSGConf::Action::RestartDaemon->new(
			name           => 'sendmail',
			pidfile        => $psgconf->sendmail_options->{'confPID_FILE'},
			filename       => 
				[ '/etc/mail/sendmail.cf', '/etc/mail/trusted-users' ]
		),
		PSGConf::Action::RestartDaemon->new(
			name           => 'msp',
			pidfile        => '/var/spool/clientmqueue/sm-client.pid',
			filename       => 
				[ '/etc/mail/submit.cf', '/etc/mail/trusted-users' ]
		),
		PSGConf::Action::RunCommand->new(
			name		=> 'sendmail alias file',
			command		=> 'newaliases',
			filename	=> $alias_file
		)
	);

	$access_map = $psgconf->sendmail_access_map;
	if (%$access_map
	    && exists($smfeatures->{access_db}))
	{
		$map_type = $smfeatures->{access_db};
		$map_type =~ s/\s.*//;
		$map_type = $smopts->{DATABASE_MAP_TYPE}
			if ($map_type eq 'DATABASE_MAP_TYPE');

		$map_path = $smfeatures->{access_db};
		$map_path = (split(/\s+/, $map_path))[-1];

		$psgconf->register_actions(
			PSGConf::Action::GenerateFile::sendmail_map->new(
				name		=> $map_path,
				description	=> 'sendmail access map',
				entries		=> $access_map
			),
			PSGConf::Action::RunCommand->new(
				name		=> 'sendmail access map',
				command		=> $psgconf->sendmail_makemap_command
						   . ' '
						   . $map_type
						   . ' '
						   . $map_path
						   . ' < '
						   . $map_path,
				filename	=> $map_path
			)
		);
	}

	$mailertable = $psgconf->sendmail_mailertable;
	if (%$mailertable
	    && exists($smfeatures->{mailertable}))
	{
		$map_type = $smfeatures->{mailertable};
		$map_type =~ s/\s.*//;
		$map_type = $smopts->{DATABASE_MAP_TYPE}
			if ($map_type eq 'DATABASE_MAP_TYPE');

		$map_path = $smfeatures->{mailertable};
		$map_path = (split(/\s+/, $map_path))[-1];

		$psgconf->register_actions(
			PSGConf::Action::GenerateFile::sendmail_map->new(
				name		=> $map_path,
				description	=> 'sendmail mailertable',
				entries		=> $mailertable
			),
			PSGConf::Action::RunCommand->new(
				name		=> 'sendmail mailertable',
				command		=> $psgconf->sendmail_makemap_command
						   . ' '
						   . $map_type
						   . ' '
						   . $map_path
						   . ' < '
						   . $map_path,
				filename	=> $map_path
			)
		);
	}
}


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self, %smopts);

	$self = {};
	bless($self, $class);

	%smopts = (
		'QUEUE_DIR'	=> '/var/spool/mqueue',
		'ALIAS_FILE'	=> '/etc/mail/aliases'
	);

	$psgconf->register_data(
		'sendmail_m4_command'	=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/usr/bin/m4'
					),
		'sendmail_m4_dir'	=> PSGConf::Data::String->new(
						'value_abspath' => 1,
						'value' => '/usr/share/sendmail'
					),
		'sendmail_path'		=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/usr/sbin/sendmail'
					),
		'sendmail_makemap_command'
					=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/usr/sbin/makemap'
					),
		'sendmail_enable'	=> PSGConf::Data::Boolean->new(),
		'sendmail_packages'	=> PSGConf::Data::List->new(),
		'sendmail_access_map'	=> PSGConf::Data::Hash->new(),
		'sendmail_mailertable'	=> PSGConf::Data::Hash->new(),
		'sendmail_aliases'	=> PSGConf::Data::Hash->new(),
		'sendmail_features'	=> PSGConf::Data::Hash->new(
						'value_optional' => 1
					),
		'sendmail_literal'	=> PSGConf::Data::String->new(),
		'sendmail_masquerade_as'
					=> PSGConf::Data::String->new(),
		'sendmail_exposed_users'	=> PSGConf::Data::List->new(),
		'sendmail_mta_user'	=> PSGConf::Data::String->new(
						value => 'root'
					),
		'sendmail_mta_group'	=> PSGConf::Data::String->new(
						value => 'daemon'
					),
		'sendmail_msa_user'	=> PSGConf::Data::String->new(
						value => 'smmsp'
					),
		'sendmail_queue_mode'	=> PSGConf::Data::Integer->new(
						value => 0755
					),
		'sendmail_msa_uid'	=> PSGConf::Data::Integer->new(
						value => 25
					),
		'sendmail_msa_group'	=> PSGConf::Data::String->new(
						value => 'smmsp'
					),
		'sendmail_msa_gid'	=> PSGConf::Data::Integer->new(
						value => 25
					),
		'sendmail_ostype'	=> PSGConf::Data::String->new(),
		'sendmail_options'	=> PSGConf::Data::Hash->new(
						'value' => \%smopts
					),
		'sendmail_trusted_users'
					=> PSGConf::Data::Hash->new(
						'value_optional' => 1
					),
		'sendmail_queue_groups'	=> PSGConf::Data::Hash->new(
						'value_type' => 'HASH'
					),
		'sendmail_queue_qf_subdirs'
					=> PSGConf::Data::Boolean->new(),
		'sendmail_num_queues_per_group'
					=> PSGConf::Data::Integer->new(
						value	=> 5
					),
		'sendmail_use_persistent_queue_runners'
					=> PSGConf::Data::Boolean->new()
	);

	$psgconf->register_policy($self,
		sm_default_pidfile	=> '_policy_default_pidfile',
		sm_default_ct_file	=> '_policy_default_ct_file',
		sm_default_aliases	=> '_policy_default_aliases',
		sm_add_rc_scripts	=> '_policy_add_rc_scripts',
		sm_add_packages	=> '_policy_add_packages',
		sm_add_user		=> '_policy_add_user'
	);

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::sendmail - psgconf control class for sendmail

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::sendmail

=head1 DESCRIPTION

The B<PSGConf::Control::sendmail> module provides a B<psgconf> control object
for configuring C<sendmail>.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<sendmail_path>

A B<PSGConf::Data::String> object that contains the absolute path to the
C<sendmail> binary.  The default is C</usr/sbin/sendmail>.

=item I<sendmail_makemap_command>

A B<PSGConf::Data::String> object that contains the C<makemap> command to
be used to generate map files.  The default is C</usr/sbin/makemap>.

=item I<sendmail_enable>

A B<PSGConf::Data::Boolean> object that indicates whether C<sendmail>
should be configured.

=item I<sendmail_packages>

A B<PSGConf::Data::List> object that lists all packages to install.

=item I<sendmail_aliases>

A B<PSGConf::Data::Hash> object that contains entries for the
F<aliases> file.

=item I<sendmail_access_map>

A B<PSGConf::Data::Hash> object that contains entries for the
access map.

=item I<sendmail_mailertable>

A B<PSGConf::Data::Hash> object that contains entries for the
mailertable.

=item I<sendmail_m4_command>

A B<PSGConf::Data::String> object that contains the C<m4> command to
be used by the B<PSGConf::Action::GenerateFile::sendmail_cf> object's
check() method.  The default is C</usr/bin/m4>.

=item I<sendmail_m4_dir>

A B<PSGConf::Data::String> object that contains the absolute path to
the directory containing C<sendmail>'s C<m4> macros.  The default is
F</usr/share/sendmail>.

=item I<sendmail_features>

A B<PSGConf::Data::Hash> object that contains the C<FEATURE()> macros
for the F</etc/mail/sendmail.mc> file.  The hash key is the name of the
feature, and the optional value is the argument.  If the argument is not
enclosed in C<`'> quotes, they will be added.

=item I<sendmail_mta_user>

A B<PSGConf::Data::String> object that contains the name of the account
to own the sendmail queue directories and run the m4 command to build 
F<sendmail.cf> files.  Defaults to C<root>.

=item I<sendmail_mta_group>

A B<PSGConf::Data::String> object that contains the group name to own
the sendmail queue directories.  Defaults to C<daemon>.

=item I<sendmail_msa_user>

A B<PSGConf::Data::String> object that contains the login for the 
Sendmail Submission User.  Defaults to C<smmsp>.

=item I<sendmail_msa_uid>

A B<PSGConf::Data::Integer> object that contains the recommended uid for
the B<sendmail_msa_user> account.  Defaults to 25.

=item I<sendmail_msa_group>

A B<PSGConf::Data::String> object that contains the group name for the 
Sendmail Submission User.  Defaults to C<smmsp>.

=item I<sendmail_msa_gid>

A B<PSGConf::Data::Integer> object that contains the recommended gid for 
the B<sendmail_msa_user> account.  Defaults to 25.

=item I<sendmail_queue_mode>

A B<PSGConf::Data::Integer> object that contains the mode of the queue
directories.  The default is 0755.

=item I<sendmail_literal>

A B<PSGConf::Data::String> object that contains literal text to add to
the end of the F</etc/mail/sendmail.mc> file.

=item I<sendmail_masquerade_as>

A B<PSGConf::Data::String> object that contains a hostname for
C<sendmail> to masquerade as.

=item I<sendmail_exposed_users>

A B<PSGConf::Data::List> object that contains list of users to have
C<sendmail> expose, even when I<sendmail_masquerade_as> is set.

=item I<sendmail_ostype>

A B<PSGConf::Data::String> object that contains the value for the
C<OSTYP> feature macro for the sendmail mc file.

=item I<sendmail_options>

A B<PSGConf::Data::Hash> object that contains the macros to be
C<define()>ed in the F</etc/mail/sendmail.mc> file.  By default,
C<QUEUE_DIR> is set to F</var/spool/mqueue>, and C<ALIAS_FILE> is set to
F</etc/mail/aliases>.

=item I<sendmail_trusted_users>

A B<PSGConf::Data::Hash> object whose keys are the list of C<sendmail>'s
trusted users.

=item I<sendmail_queue_groups>

A B<PSGConf::Data::Hash> object that contains the queue groups to be
configured in the F</etc/mail/sendmail.mc> file.  The hash key is the
name of the queue group, and the value is an anonymous hash containing
the attributes for that queue group.

=item I<sendmail_queue_qf_subdirs>

A B<PSGConf::Data::Boolean> object that indicates whether the C<qf>,
C<df>, and C<xf> queue subdirectories should be created.

B<NOTE:> If you enable this on a system where C<sendmail> is already
running, you must stop C<sendmail>, go to each queue directory and move
all C<qf?*> files to the C<qf> subdirectory and all C<df?*> files to
the C<df> subdirectory, and then restart C<sendmail>.  Failure to do
this can result in lost messages!

=item I<sendmail_num_queues_per_group>

A B<PSGConf::Data::Integer> object that indicates the number of queues
to be created for each queue group.  This is only relevant if either
I<sendmail_queue_groups> is set or if the value of the C<QUEUE_DIR>
entry in I<sendmail_options> ends with a C<*>.

=item I<sendmail_use_persistent_queue_runners>

A B<PSGConf::Data::Boolean> object that indicates whether persistent
queue runners should be used.

=back

The constructor also registers the following policy methods:

=over 4

=item I<sm_add_rc_scripts>

Sets up the C<sendmail> RC script.  This is done using the I<rc_scripts>
data object, which are provided by the B<PSGConf::Control::InitScripts> module.

=item I<sm_add_user>

Adds the C<smmsp> user and group using the I<user_info> and
I<group_info> data objects, which are provided by the
B<PSGConf::Control::Users> module.

=item I<sm_add_packages>

Requests that the C<sendmail> package be installed.  This is done using
the I<pkg_install_list> object, which is provided by the
B<PSGConf::Control::Packages> module.

=item I<sm_default_pidfile>

If the I<sendmail_options> object does not contain an entry for
C<confPID_FILE>, set it to F<I<pidfile_dir>/sendmail.pid>.  (The
I<pidfile_dir> object is provided by the B<PSGConf::Control::Core>
module.)

=item I<sm_default_ct_file>

If the I<sendmail_trusted_users> object has been set and the
I<sendmail_options> object does not contain an entry for C<confCT_FILE>,
set C<confCT_FILE> to C<-o /etc/mail/trusted-users> and add an entry
for C<use_ct_file> to the I<sendmail_features> object.

=item I<sm_default_aliases>

If there is no entry for C<postmaster> in the I<sendmail_aliases> data
object, set it to C<root>.

If there is no entry for C<MAILER-DAEMON> in the I<sendmail_aliases> data
object, set it to C<postmaster>.

If there is no entry for C<nobody> in the I<sendmail_aliases> data
object, set it to C</dev/null>.

=back

=item decide()

If I<sendmail_enable> is on, instantiates and registers action objects,
as follows:

=over 4

=item *

Registers B<PSGConf::Action::MkDir> action objects to create
any needed queue directories.  The queue directories are
calculated based on the values of the I<sendmail_queue_groups> and
I<sendmail_num_queues_per_group> data objects and the C<QUEUE_DIR>
entry in the I<sendmail_options> object.

=item *

Registers a B<PSGConf::Action::MkDir> action object to create the
F</var/spool/clientmqueue> directory.  The directory will be given mode
0770 with the group ID of the C<smmsp> group.  (The group ID is obtained
from the I<group_info> object, which is provided by the
B<PSGConf::Control::Users> module.)

=item *

Registers B<PSGConf::Action::GenerateFile::sendmail_mc> action objects
to create the F</etc/mail/sendmail.mc> and F</etc/mail/submit.mc> files.

=item *

Registers B<PSGConf::Action::GenerateFile::sendmail_cf> action objects
to create the F</etc/mail/sendmail.cf> and F</etc/mail/submit.cf> files.

=item *

Registers a B<PSGConf::Action::GenerateFile::sendmail_trusted_users>
object to create the F</etc/mail/trusted-users> file.

=item *

Registers a B<PSGConf::Action::GenerateFile::sendmail_aliases> action
object to create the F<aliases> file and a B<PSGConf::Action::RunCommand>
action object to run C<newaliases>.  The path for the file is taken
from the C<ALIAS_FILE> entry in the I<sendmail_options> object.
The C<ALIAS_FILE> value will be truncated at the first C<,> character.

=item *

If C<confHOST_STATUS_DIRECTORY> is set in the I<sendmail_options> object,
registers a B<PSGConf::Action::MkDir> action object to create the
specified directory.

=item *

If I<sendmail_access_map> is set and I<sendmail_features> contains an
entry called C<access_db>, registers a
B<PSGConf::Action::GenerateFile::sendmail_map> object to create the
access map and a B<PSGConf::Action::RunCommand> object to run C<makemap>
on the new map file.

=item *

If I<sendmail_mailertable> is set, registers a
B<PSGConf::Action::GenerateFile::sendmail_map> object to create the
access map and a B<PSGConf::Action::RunCommand> object to run C<makemap>
on the new map file.

=back

=head1 SEE ALSO

sendmail(8)

newaliases(1)

m4(1)

makemap(1)

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::sendmail_aliases>

L<PSGConf::Action::GenerateFile::sendmail_mc>

L<PSGConf::Action::GenerateFile::sendmail_cf>

L<PSGConf::Action::GenerateFile::sendmail_trusted_users>

L<PSGConf::Action::GenerateFile::sendmail_map>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::RunCommand>

L<PSGConf::Action::RestartDaemon>

L<PSGConf::Control::Core>

L<PSGConf::Control::InitScripts>

L<PSGConf::Control::Packages>

L<PSGConf::Control::Users>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Hash>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<psgconf-intro>

=cut

