
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  sudo.pm - sudo module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::sudo;

use strict;

use PSGConf::Action::GenerateFile::sudoers;
use PSGConf::Action::Remove;
use PSGConf::Data::Boolean;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;

use PSGConf::Util;


###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		'sudo_enable'		=> PSGConf::Data::Boolean->new(
						value => 0
				        ),	
		'sudo'			=> PSGConf::Data::Hash->new(
						'value_type' => 'HASH'
					),
		'sudoers_path'		=> PSGConf::Data::String->new(
						value_abspath => 1,
						value => '/etc/sudoers'
					),
		'sudo_defaults'		=> PSGConf::Data::String->new(),
		'sudo_packages'		=> PSGConf::Data::List->new(),
		'sudo_nopasswd'		=> PSGConf::Data::Boolean->new(
						value => 1
					)
	);

	$psgconf->register_policy($self,
		sudo_add_packages	=> '_policy_add_packages'
	);

	return $self;
}


###############################################################################
###  policy method
###############################################################################

sub _policy_add_packages
{
	my ($self, $psgconf) = @_;

	PSGConf::Util::_add_packages(
		$psgconf, 
		$psgconf->sudo_enable,
		@{$psgconf->sudo_packages}
	);
}


###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;

	if ( !$psgconf->sudo_enable && ! %{$psgconf->sudo})
	{
		$psgconf->register_actions(
			PSGConf::Action::Remove->new(
				'name'		=> $psgconf->sudoers_path
			)
		);
		return;
	}

	$psgconf->register_actions(
		PSGConf::Action::GenerateFile::sudoers->new(
			'name'		=> $psgconf->sudoers_path,
			'description'	=> 'sudo access control file',
			'mode'		=> 0440,
			'hostname'	=> $psgconf->hostname,
			'defaults'	=> $psgconf->sudo_defaults,
			'sudo'		=> $psgconf->sudo,
			'no_passwd'	=> $psgconf->sudo_nopasswd,
		)
	);
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::sudo - psgconf control class for sudo

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::sudo

=head1 DESCRIPTION

The B<PSGConf::Control::sudo> module provides a B<psgconf> control object
for configuring C<sudo>.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<sudo_enable>

A B<PSGConf::Data::Boolean> object indicating if we should use C<sudo>
on the system.  The default is no.

=item I<sudo>

A B<PSGConf::Data::Hash> object containing the permission entries for
F</etc/sudoers>.  The hash key is the command.  The value is a reference
to a hash whose key is the user to run the command as, and whose value
is a reference to yet another hash whose keys are the list of users who
are allowed to run the command as that user.

=item I<sudoers_path>

A B<PSGConf::Data::String> object containing the filename for the F<sudoers>
file.

=item I<sudo_defaults>

A B<PSGConf::Data::String> object containing the arguments for the
C<Defaults> directive in F</etc/sudoers>.

=item I<sudo_nopasswd>

A B<PSGConf::Data::Boolean> object indicating whether the user should
be required to enter a password when using C<sudo>.

=item I<sudo_packages>

A B<PSGConf::Data::List> object containing the packages to install.

=back

The constructor also registers the following policy method:

=over 4

=item I<sudo_add_packages>

If I<sudo> is defined, adds the C<sudo> package to the I<pkg_install_list>
object, which is provided by the B<PSGConf::Control::Packages> module.

=back

=item decide()

If I<sudo> is defined, it registers a
B<PSGConf::Action::GenerateFile::sudoers> action object to create the
C<sudo> config file.  Otherwise, registers a B<PSGConf::Action::Remove>
object to remove the C<sudo> config file.

=back

=head1 SEE ALSO

sudoers(4)

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::sudoers>

L<PSGConf::Action::Remove>

L<PSGConf::Control::Packages>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Hash>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<psgconf-intro>

=cut

