
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  syslog.pm - syslog module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Control::syslog;

use strict;

use PSGConf::Action::GenerateFile::syslog_conf;
use PSGConf::Action::MkDir;
use PSGConf::Action::CreateFile;
use PSGConf::Action::RestartDaemon;
use PSGConf::Data::Boolean;
use PSGConf::Data::Hash;
use PSGConf::Data::String;
use PSGConf::Util;


###############################################################################
###  policy methods
###############################################################################

sub _policy_add_packages
{
     my ($self, $psgconf) = @_;

     PSGConf::Util::_add_packages(
          $psgconf,
          $psgconf->syslog_enable,
          @{$psgconf->syslog_packages}
     );
}

### set default location for syslog_dir
sub _policy_default_dir
{
	my ($self, $psgconf) = @_;

	return
		if (! $psgconf->syslog_enable);

	$psgconf->data_obj('syslog_dir')->prepend($psgconf->log_dir . '/')
		if (substr($psgconf->syslog_dir, 0, 1) ne '/');
}


### set absolute path to log files
sub _policy_canonify_logfiles
{
	my ($self, $psgconf) = @_;
	my ($syslog, $syslog_dir);
	my ($log, $new_log);

	return
		if (! $psgconf->syslog_enable);

	$syslog = $psgconf->syslog;
	$syslog_dir = $psgconf->syslog_dir;

	foreach $log (keys %$syslog)
	{
		if (substr($log, 0, 1) ne '/' && substr($log, 0, 1) ne '@') 
		{
			$new_log = "$syslog_dir/$log";
			$syslog->{$new_log} = $syslog->{$log};
			delete $syslog->{$log};
		}
	}
}


### handle platforms with "authpriv" facility
sub _policy_add_authpriv_facility
{
	my ($self, $psgconf) = @_;
	my ($href, $target);

	return
		if (! $psgconf->syslog_enable
		    || $psgconf->platform =~ m/solaris/);

	foreach $href ($psgconf->syslog, $psgconf->syslog_literal)
	{
		foreach $target (sort keys %$href)
		{
			$href->{$target} =~ s/\bauth\b/$&,authpriv/;
		}
	}
}


### expand USED_FACILITIES token
sub _policy_expand_used_facilities
{
	my ($self, $psgconf) = @_;
	my ($syslog, $log, $levels, $facilities, %used_facilities, $default);

	return
		if (! $psgconf->syslog_enable);

	$syslog = $psgconf->syslog;

	foreach $log (keys %$syslog)
	{
		if ($syslog->{$log} =~ m/USED_FACILITIES/)
		{
			$default = $log;
			next;
		}

		foreach $levels (split(';', $syslog->{$log}))
		{
			$levels =~ s/\.\w+//;
			map { $used_facilities{$_}++; } split(',', $levels);
		}
	}

	if (defined($default))
	{
		$facilities = join(',', sort keys %used_facilities);
		$syslog->{$default} =~ s/USED_FACILITIES/$facilities/;
	}
}


sub _policy_add_remote_servers 
{
	my ($self, $psgconf) = @_;
	my ($syslog);

	$syslog = $psgconf->syslog;

	map {
		$syslog->{ '@' . $_ } = '*.info';
	} @{$psgconf->syslog_servers};
}

sub _policy_enable_rc_scripts
{
	my ($self, $psgconf) = @_;

     $psgconf->rc_scripts->{syslogd}->{state} = 'enable'
          if ( $psgconf->syslog_enable );
}


###############################################################################
###  decide() method
###############################################################################

sub decide
{
	my ($self, $psgconf) = @_;
	my ($file);

	return
		if (! $psgconf->syslog_enable);

	$psgconf->register_actions(
		PSGConf::Action::GenerateFile::syslog_conf->new(
			'name'		=> $psgconf->syslog_file,
			'description'	=> 'syslogd configuration file',
			'syslog'	=> {
						%{$psgconf->syslog},
						%{$psgconf->syslog_literal}
					}
		),
		PSGConf::Action::MkDir->new(
			'name'		=> $psgconf->syslog_dir
		)
	);

	foreach $file ( sort keys %{$psgconf->syslog}) {
		if (substr($file, 0, 1) eq '/') {
			$psgconf->register_actions(
				PSGConf::Action::CreateFile->new(
					'name'	=> $file
				)
			);
		}
	}

	### Make sure we restart syslogd after we create all the files 
	$psgconf->register_actions(
		PSGConf::Action::RestartDaemon->new(
			name           => 'syslogd',
			filename       => [ $psgconf->syslog_dir, 
							$psgconf->syslog_file ],
			pidfile		=> $psgconf->pidfile_dir . '/syslog.pid'
		)
	);
}



###############################################################################
###  Constructor
###############################################################################

sub new
{
	my ($class, $psgconf) = @_;
	my ($self);

	$self = {};
	bless($self, $class);

	$psgconf->register_data(
		'syslog_enable'		=> PSGConf::Data::Boolean->new(
						value => 1
					),
		'syslog'		=> PSGConf::Data::Hash->new(),
		'syslog_file'		=> PSGConf::Data::String->new(
						'value' => '/etc/syslog.conf'
					),
		'syslog_dir'		=> PSGConf::Data::String->new(
						'value' => 'syslog'
					),
		'syslog_literal'	=> PSGConf::Data::Hash->new(),
		'syslog_packages'	=> PSGConf::Data::List->new(),
		'syslog_servers'	=> PSGConf::Data::List->new()
	);

	$psgconf->register_policy($self,
		syslog_default_dir	=> '_policy_default_dir',
		syslog_canonify_logfiles
					=> '_policy_canonify_logfiles',
		syslog_add_authpriv_facility
					=> '_policy_add_authpriv_facility',
		syslog_expand_used_facilities
					=> '_policy_expand_used_facilities',
		syslog_enable_rc_scripts
					=> '_policy_enable_rc_scripts',
		syslog_add_packages	=> '_policy_add_packages',
		syslog_add_remote_servers => '_policy_add_remote_servers'
	);

	return $self;
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::syslog - psgconf control class for syslog

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::syslog

=head1 DESCRIPTION

The B<PSGConf::Control::syslog> module provides a B<psgconf> control object
for configuring C<syslogd>.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<syslog_enable>

A B<PSGConf::Data::Boolean> object that indicates whether C<syslog>
should be configured.  The default is true.

=item I<syslog_packages>

A B<PSGConf::Data::List> object that lists all packages to install.

=item I<syslog>

A B<PSGConf::Data::Hash> object that contains log file entries for
F<syslog.conf>.  The hash key is the name of the file to create,
and the value is the facility and priority specification.

=item I<syslog_file>

A B<PSGConf::Data::String> object that contains the path for the
syslog config file.  The default is F</etc/syslog.conf>.

=item I<syslog_dir>

A B<PSGConf::Data::String> object that contains the path to the
directory where log files will be stored.  If not an absolute path, it
is relative to the value of the I<log_dir> object, which is supplied by
the B<PSGConf::Control::Core> module.

=item I<syslog_servers>

A B<PSGConf::Data::List> object that lists all remote syslog servers
to send data to.

=item I<syslog_literal>

A B<PSGConf::Data::Hash> object that contains non-file entries for
I<syslog_file>.  The hash key is the target (e.g., a login name or
a C<*>), and the value is the facility and priority specification.

=back

The constructor also registers the following policy methods:

=over 4

=item I<syslog_default_dir>

If I<syslog_dir> is not set to an absolute path, prepend the value of
the I<log_dir> object, which is provided by the
B<PSGConf::Control::Core> module.

=item I<syslog_canonify_logfiles>

For each entry in the I<syslog> object that is not an absolute path,
prepend the value of the I<syslog_dir> object.

=item I<syslog_add_authpriv_facility>

On platforms other than Solaris, add the C<authpriv> facility to any
entry in I<syslog> or I<syslog_literal> that already specifies the
C<auth> facility.

=item I<syslog_expand_used_facilities>

Expands the token C<USED_FACILITIES> in the I<syslog> data object with
the list of facilities used by other entries in I<syslog>.

=item I<syslog_enable_rc_scripts>

Modifies the I<rc_scripts> data object (provided by
B<PSGConf::Control::InitScripts>) to enable C<syslogd>.

=back

=item decide()

If the I<syslog_enable> data object is set, instantiates and registers
action objects, as follows:

=over 4

=item *

Registers a new B<PSGConf::Action::GenerateFile::syslog_conf> object to
create I<syslog_file>.

=item *

Registers a new B<PSGConf::Action::MkDir> object to create the
I<syslog_dir> directory.

=item *

Registers new B<PSGConf::Action::CreateFile> objects for each file
in the I<syslog> object.

=back

=head1 SEE ALSO

syslog.conf(4)

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::syslog_conf>

L<PSGConf::Action::MkDir>

L<PSGConf::Action::CreateFile>

L<PSGConf::Action::RestartDaemon>

L<PSGConf::Control::Core>

L<PSGConf::Control::InitScripts>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Hash>

L<PSGConf::Data::String>

L<PSGConf::Util>

L<psgconf-intro>

=cut

