
###
###  Copyright 2000-2005 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  PSGConf::Util - utility functions for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Util;

use strict;

use File::Find;
use Net::hostent;
use POSIX;
use Socket;

our @ISA = qw(Exporter);

our @EXPORT = qw(
	get_addrs
	chown_recursive
	platform_name
);


###############################################################################
###  _add_packages - Generic method to add a list of packages to the
###                  pkg_install_list
###############################################################################
sub _add_packages {
	my ($psgconf, $enable, @packages) = @_;

     return if (! $enable);

     map {
          $psgconf->pkg_install_list->{$_} = undef
               if (! exists($psgconf->pkg_install_list->{$_}));
     } @packages;
}

###############################################################################
###  _expand_tokens - Generic method to expand tokens of the form
###                   %{directive} to their value, only really works
###                   with String/Integer data types.
###############################################################################
sub _expand_tokens
{
     my ($psgconf, $text) = @_;
     my ($start_idx, $newtext, $end_idx, $token, $val);

     $start_idx = 0;
	$newtext = $text;
     while ($start_idx != -1)
     {
          $start_idx = index($text, "%{", $start_idx);
          next
               if ($start_idx == -1);
          $end_idx = index($text, "}", $start_idx);

          $token = substr($text, $start_idx + 2,
                         $end_idx - $start_idx - 2);
		$val = $psgconf->$token;
		$token = "%{" . $token . "}";

		$newtext =~ s/$token/$val/g;

          $start_idx = $end_idx;
     }
	return $newtext;
}

###############################################################################
###  _maxstr - Compare 2 different strings
###############################################################################
###
### I was using the List::Util::maxstr from perl 5.8 modules, but
### that does not exist in perl 5.6 releases.  Hence not to have
### another dependancy, write my own :(
###
sub _maxstr {
     my ($a, $b) = @_;
     return ($a gt $b) ? $a : $b;
}

###############################################################################
###  get_addrs(FQDN) - get all IP addresses associated with FQDN
###############################################################################

sub get_addrs
{
	my ($psgconf, $fqdn) = @_;
	my ($hent, @res);

	### First see if we have the IP address from our configs
	if ( defined $psgconf->host_addrs->{$fqdn} ) {
		@res = $psgconf->host_addrs->{$fqdn};

	} else {
		### Now check to make sure we do not have a host_alias set either
		map { 
			if ( exists $psgconf->host_aliases->{$_}->{$fqdn} ) {
				if ( defined $psgconf->host_addrs->{$_} ) {
					@res = $psgconf->host_addrs->{$_};

				### Save new name to look up in DNS
				} else {
					$fqdn = $_;
				}
			}
		} keys %{$psgconf->host_aliases};

		if ( ! scalar @res ) {
			if ($hent = gethostbyname($fqdn)) {
				@res = sort map { inet_ntoa($_) } @{$hent->addr_list};
			}
		}
	}

	die "unknown host \"$fqdn\""
		if (! scalar @res);

	return @res;
}


###############################################################################
###  RunCommand(command) - Run a command using system(3).
###############################################################################

sub RunCommand
{  
     my ($cmd, $nowarn) = @_;
	my ($rc);
     
	if (($rc=system($cmd))) {

		### If we have a non zero return code, then
		### look at errno ($?) to determine what happened
		if ( $rc == -1 || WIFEXITED($?) ) {
         		warn "\t!!! command '$cmd' failed: $!\n"
				if ( ! $nowarn );
			$rc = -1;

		### OK, so the program was killed by a signal.
		} elsif ( WIFSIGNALED($?) ) {
         		warn "\t!!! command '$cmd' terminated on signal " 
				. WTERMSIG($?)
				. ($? & 128)? ", with": ", without"
				. " coredump\n"
				if ( ! $nowarn );
			$rc= WTERMSIG($?);

		### Punt, we had some other type of error
		} elsif ( $? ) {
         		warn "\t!!! command '$cmd' failed: $!\n"
				if ( ! $nowarn );
			$rc = $? >> 8;
		}
	}
 
     return $rc;
}


###############################################################################
###  chown_recursive(DIR, UID, GID) - recursive chown DIR to UID:GID
###############################################################################

sub chown_recursive
{
	my ($dir, $uid, $gid) = @_;
	my ($retval);

	print $0 . ": chown'ing $dir\n";

	$retval = 1;
	find(sub {
		if (!chown($uid, $gid, $File::Find::name))
		{
			warn "\n\t!!! chown(\"$File::Find::name\"): $!\n";
			$retval = 0;
		}
	}, $dir);

	return $retval;
}


###############################################################################
###  platform_name() - return platform name for current host
###############################################################################

sub platform_name
{
	my ($sysname, $nodename, $release, $version, $machine);
	my ($platform, @suffixes);

	($sysname, $nodename, $release, $version, $machine) = POSIX::uname();

	if ($sysname eq 'AIX')
	{
		my $os_level;

		$os_level = `oslevel`;
		chomp($os_level);
		$os_level =~ s/^(\d+\.\d+\.\d+)\.\d+$/$1/;

		$platform = "rs6000-aix$os_level";
	}

	elsif ($sysname eq 'HP-UX')
	{
		### assume pa, since we can't get at _SC_CPU_VERSION
		$release =~ s/^[^\d]+//;
		$platform = "pa-hpux$release";
	}

	elsif ($sysname =~ m/^IRIX.*/)
	{
		### assume mips, since we don't have an interface to sysinfo()
		$platform = "mips-irix$release";
	}

	elsif ($sysname eq 'Linux')
	{
		### only save the first two digits of the kernel version
		$release =~ s/^(\d+\.\d+)\..*$/$1/;
		$platform = (($machine =~ m/^i\d86$/) ? 'ix86' : $machine);

		### set the other arches we can support (i686 will support
		### i386 to i686 as well athlon).
		if ( $platform eq 'ix86' ) {
			my ($x);
			grep (/i([3-9])86/o && ($x=$1), $machine);
			for (my($i) = 3; $i <= $x; $i++) {
				push @suffixes, "i${i}86";
			}

			### FIXME.  Need to add the athlon arch tag
			### for those systems (output of uname -p).
		}

		$platform .= "-linux$release";
	}

	elsif ($sysname eq 'FreeBSD')
	{
		### only save the major number of the kernel version
		$release =~ s/^(\d+)\..*$/$1/;
		$platform = (($machine =~ m/^i\d86$/) ? 'ix86' : $machine);
		$platform .= "-freebsd$release";
	}

	elsif ($sysname eq 'SunOS')
	{
		my $bits;

		$platform = (($machine eq 'i86pc') ? 'ix86' : 'sparc');
		my ($maj, $min) = split (/\./, $release);
		$platform .= '-' . (($maj >= 5) ? 'solaris' : 'sunos');
		$platform .= '2.'
			if ($min <= 6);
		$platform .= $min;

		### set 32-bit vs 64-bit suffix
		$bits = `isainfo -b`;
		chomp($bits);
		@suffixes = ("${bits}bit");
	}

	elsif ($sysname eq 'Darwin')
	{
		$platform = (($machine eq 'Power Macintosh') ? 'powerpc' : $machine);
		$platform .= "-darwin$release";
	}

	return ($platform, @suffixes);
}


1;

